
/*
	File:		LayoutFeatureConstants.h
	
	Contains:	Constants identifying the feature types and selectors currently
				defined in the Font Feature Registry (Link: FONTREGISTRY).
	
	Written By:	Dave Opstad
	
	Copyright:	1992-1995 by Apple Computer, Inc.  All rights reserved.
	
	Change History (most recent first):
	
		 <2>	 5/25/95	DGO		Added 3 new JIS character shape selectors, at the request of
									Canon (who are developing new GX fonts).
		<4>		950525	DGO		Added three selectors for JIS choices.
		<3>		1/9/95	JD		First checked in to NRBuild system.
		<2>		940928	DGO		Added textSpacingType and its selectors.
		<1>		921201	DGO		Created module.

*/

#ifndef __LAYOUTFEATURECONSTANTS__
#define __LAYOUTFEATURECONSTANTS__

#ifdef __cplusplus
extern "C" {
#endif

/* for compatibility with old headers */
#define layoutFeatureConstantsIncludes

/* The following values can be used to set gxRunFeature values. Note that unless the
	feature is defaulted differently in different fonts, the zero value for the
	selectors represents the default value. */

enum {

	allTypographicFeaturesType = 0,

		allTypeFeaturesOnSelector			= 0,
		allTypeFeaturesOffSelector			= 1,

	ligaturesType = 1,

		requiredLigaturesOnSelector			= 0,
		requiredLigaturesOffSelector		= 1,
		commonLigaturesOnSelector			= 2,
		commonLigaturesOffSelector			= 3,
		rareLigaturesOnSelector				= 4,
		rareLigaturesOffSelector			= 5,
		logosOnSelector						= 6,
		logosOffSelector					= 7,
		rebusPicturesOnSelector				= 8,
		rebusPicturesOffSelector			= 9,
		diphthongLigaturesOnSelector		= 10,
		diphthongLigaturesOffSelector		= 11,

	cursiveConnectionType = 2,

		unconnectedSelector					= 0,
		partiallyConnectedSelector			= 1,
		cursiveSelector						= 2,

	letterCaseType = 3,

		upperAndLowerCaseSelector			= 0,
		allCapsSelector						= 1,
		allLowerCaseSelector				= 2,
		smallCapsSelector					= 3,
		initialCapsSelector					= 4,
		initialCapsAndSmallCapsSelector		= 5,

	verticalSubstitutionType = 4,

		substituteVerticalFormsOnSelector	= 0,
		substituteVerticalFormsOffSelector	= 1,

	linguisticRearrangementType = 5,

		linguisticRearrangementOnSelector	= 0,
		linguisticRearrangementOffSelector	= 1,

	numberSpacingType = 6,

		monospacedNumbersSelector			= 0,
		proportionalNumbersSelector			= 1,

	appleReserved1Type = 7,

	smartSwashType = 8,

		wordInitialSwashesOnSelector		= 0,
		wordInitialSwashesOffSelector		= 1,
		wordFinalSwashesOnSelector			= 2,
		wordFinalSwashesOffSelector			= 3,
		lineInitialSwashesOnSelector		= 4,
		lineInitialSwashesOffSelector		= 5,
		lineFinalSwashesOnSelector			= 6,
		lineFinalSwashesOffSelector			= 7,
		nonFinalSwashesOnSelector			= 8,
		nonFinalSwashesOffSelector			= 9,

	diacriticsType = 9,

		showDiacriticsSelector				= 0,
		hideDiacriticsSelector				= 1,
		decomposeDiacriticsSelector			= 2,

	verticalPositionType = 10,

		normalPositionSelector				= 0,
		superiorsSelector					= 1,
		inferiorsSelector					= 2,
		ordinalsSelector					= 3,

	fractionsType = 11,

		noFractionsSelector					= 0,
		verticalFractionsSelector			= 1,
		diagonalFractionsSelector			= 2,

	appleReserved2Type = 12,

	overlappingCharactersType = 13,

		preventOverlapOnSelector			= 0,
		preventOverlapOffSelector			= 1,

	typographicExtrasType = 14,

		hyphensToEmDashOnSelector			= 0,
		hyphensToEmDashOffSelector			= 1,
		hyphenToEnDashOnSelector			= 2,
		hyphenToEnDashOffSelector			= 3,
		unslashedZeroOnSelector				= 4,
		unslashedZeroOffSelector			= 5,
		formInterrobangOnSelector			= 6,
		formInterrobangOffSelector			= 7,
		smartQuotesOnSelector				= 8,
		smartQuotesOffSelector				= 9,

	mathematicalExtrasType = 15,

		hyphenToMinusOnSelector				= 0,
		hyphenToMinusOffSelector			= 1,
		asteriskToMultiplyOnSelector		= 2,
		asteriskToMultiplyOffSelector		= 3,
		slashToDivideOnSelector				= 4,
		slashToDivideOffSelector			= 5,
		inequalityLigaturesOnSelector		= 6,
		inequalityLigaturesOffSelector		= 7,
		exponentsOnSelector					= 8,
		exponentsOffSelector				= 9,

	ornamentSetsType = 16,

		noOrnamentsSelector					= 0,
		dingbatsSelector					= 1,
		piCharactersSelector				= 2,
		fleuronsSelector					= 3,
		decorativeBordersSelector			= 4,
		internationalSymbolsSelector		= 5,
		mathSymbolsSelector					= 6,

	characterAlternativesType = 17,
	
		noAlternatesSelector				= 0,
		
		/* Additional character alternatives selectors may be defined by a font. */
	
	designComplexityType = 18,

		designLevel1Selector				= 0,
		designLevel2Selector				= 1,
		designLevel3Selector				= 2,
		designLevel4Selector				= 3,
		designLevel5Selector				= 4,

	styleOptionsType = 19,

		noStyleOptionsSelector				= 0,
		displayTextSelector					= 1,
		engravedTextSelector				= 2,
		illuminatedCapsSelector				= 3,
		titlingCapsSelector					= 4,
		tallCapsSelector					= 5,

	characterShapeType = 20,

		traditionalCharactersSelector		= 0,
		simplifiedCharactersSelector		= 1,
		jis1978CharactersSelector			= 2,
		jis1983CharactersSelector			= 3,
		jis1990CharactersSelector			= 4,

	numberCaseType = 21,

		lowerCaseNumbersSelector			= 0,
		upperCaseNumbersSelector			= 1,

	textSpacingType = 22,
	
		proportionalTextSelector            = 0,
		monospacedTextSelector              = 1,

	lastFeatureType = -1
};


/*-------------------------------------------------------------------------------------*/

/* The constants provided below are for backward compatibility only.  Use of the above
 constant names is preferred. */

enum {

	typographicFeaturesType = allTypographicFeaturesType,
	
		enableEffectsOnSelector                 = allTypeFeaturesOnSelector,
		enableEffectsOffSelector                = allTypeFeaturesOffSelector,
	
	ligatureType = ligaturesType,
	
		ligatureRequiredOnSelector              = requiredLigaturesOnSelector,
		ligatureRequiredOffSelector             = requiredLigaturesOffSelector,
		ligatureCommonOnSelector                = commonLigaturesOnSelector,
		ligatureCommonOffSelector               = commonLigaturesOffSelector,
		ligatureRareOnSelector                  = rareLigaturesOnSelector,
		ligatureRareOffSelector                 = rareLigaturesOffSelector,
		ligatureLogotypeOnSelector              = logosOnSelector,
		ligatureLogotypeOffSelector             = logosOffSelector,
		ligatureRebusOnSelector                 = rebusPicturesOnSelector,
		ligatureRebusOffSelector                = rebusPicturesOffSelector,
	
/*  cursiveConnectionType = 2, */
	
		cursiveConnectionNoneSelector           = unconnectedSelector,
		cursiveConnectionPartialSelector        = partiallyConnectedSelector,
		cursiveConnectionFullSelector           = cursiveSelector,
	
	letterNumberCaseType = letterCaseType,
	
		letterNumberCaseUpperLowerSelector      = upperAndLowerCaseSelector,
		letterNumberCaseAllUpperSelector        = allCapsSelector,
		letterNumberCaseAllLowerSelector        = allLowerCaseSelector,
		letterNumberCaseSmallCapsSelector       = smallCapsSelector,
		letterNumberCaseUpperThenLowerSelector  = initialCapsSelector,
		letterNumberCaseUpperThenSmCapsSelector = initialCapsAndSmallCapsSelector,
	
/*  verticalSubstitutionType = 4, */
	
		verticalSubstitutionOnSelector          = substituteVerticalFormsOnSelector,
		verticalSubstitutionOffSelector         = substituteVerticalFormsOffSelector,
	
	rearrangementType = linguisticRearrangementType,
	
		rearrangementOnSelector                 = linguisticRearrangementOnSelector,
		rearrangementOffSelector                = linguisticRearrangementOffSelector,
	
	numberStyleType = numberSpacingType,
	
		numberStyleForColumnsSelector           = monospacedNumbersSelector,
		numberStyleProportionalSelector         = proportionalNumbersSelector,
	
/*	Note: The checkbox feature type Alternate Designs has been removed.

	Most of its settings have been divided between two new RADIO BUTTON feature types:
		Design Complexity
			Book Text, Sloped, and Chancery have been replaced by generic design levels.
		Style Options
			Engraved, Display Text, Illuminated Text, and Titling Caps have been moved.
			
	The Cursive setting has been eliminated. */
	
/*  smartSwashType = 8, */
	
		smartSwashWordInitialsOnSelector        = wordInitialSwashesOnSelector,
		smartSwashWordInitialsOffSelector       = wordInitialSwashesOffSelector,
		smartSwashWordFinalsOnSelector          = wordFinalSwashesOnSelector,
		smartSwashWordFinalsOffSelector         = wordFinalSwashesOffSelector,
		smartSwashLineInitialsOnSelector        = lineInitialSwashesOnSelector,
		smartSwashLineInitialsOffSelector       = lineInitialSwashesOffSelector,
		smartSwashLineFinalsOnSelector          = lineFinalSwashesOnSelector,
		smartSwashLineFinalsOffSelector         = lineFinalSwashesOffSelector,
		smartSwashArchaicNonFinalsOnSelector    = nonFinalSwashesOnSelector,
		smartSwashArchaicNonFinalsOffSelector   = nonFinalSwashesOffSelector,
	
/*  diacriticsType = 9, */
	
		diacriticsShowSelector                  = showDiacriticsSelector,
		diacriticsHideSelector                  = hideDiacriticsSelector,
		diacriticsSerializeSelector             = decomposeDiacriticsSelector,
	
/*  verticalPositionType = 10, */
	
		verticalPositionNormalSelector          = normalPositionSelector,
		verticalPositionSuperiorsSelector       = superiorsSelector,
		verticalPositionInferiorsSelector       = inferiorsSelector,
		verticalPositionOrdinalsSelector        = ordinalsSelector,
	
/*  fractionsType = 11, */
	
		fractionsDontFormSelector               = noFractionsSelector,
		fractionsFormVerticalSelector           = verticalFractionsSelector,
		fractionsFormDiagonalSelector           = diagonalFractionsSelector,
	
	diphthongLigaturesType = ligaturesType,
	
/*    diphthongLigaturesOnSelector            = 0,
		diphthongLigaturesOffSelector           = 1, */
	
	overlappingGlyphsType = overlappingCharactersType,
	
		overlappingGlyphsPreventOnSelector      = preventOverlapOnSelector,
		overlappingGlyphsPreventOffSelector     = preventOverlapOffSelector,
	
/*  typographicExtrasType = 14, */
	
		extrasHyphensToEmDashOnSelector         = hyphensToEmDashOnSelector,
		extrasHyphensToEmDashOffSelector        = hyphensToEmDashOffSelector,
		extrasHyphenToEnDashOnSelector          = hyphenToEnDashOnSelector,
		extrasHyphenToEnDashOffSelector         = hyphenToEnDashOffSelector,
		extrasForceUnslashedZeroOnSelector      = unslashedZeroOnSelector,
		extrasForceUnslashedZeroOffSelector     = unslashedZeroOffSelector,
		extrasMakeInterrobangOnSelector         = formInterrobangOnSelector,
		extrasMakeInterrobangOffSelector        = formInterrobangOffSelector,
	
	mathExtrasType = mathematicalExtrasType,
	
		mathHyphenToMinusOnSelector             = hyphenToMinusOnSelector,
		mathHyphenToMinusOffSelector            = hyphenToMinusOffSelector,
		mathStarToMultiplyOnSelector            = asteriskToMultiplyOnSelector,
		mathStarToMultiplyOffSelector           = asteriskToMultiplyOffSelector,
		mathSlashToDivideOnSelector             = slashToDivideOnSelector,
		mathSlashToDivideOffSelector            = slashToDivideOffSelector,
		mathInequalityLigaturesOnSelector       = inequalityLigaturesOnSelector,
		mathInequalityLigaturesOffSelector      = inequalityLigaturesOffSelector,
		mathExponentiationOnSelector            = exponentsOnSelector,
		mathExponentiationOffSelector           = exponentsOffSelector,
	
/*  ornamentSetsType = 16, */
	
		ornamentSetsNoneSelector                = noOrnamentsSelector,
		ornamentSetsDingbatsSelector            = dingbatsSelector,
		ornamentSetsPiCharactersSelector        = piCharactersSelector,
		ornamentSetsFleuronsSelector            = fleuronsSelector,
		ornamentSetsDecorativeBordersSelector   = decorativeBordersSelector,
		ornamentSetsIntlSymbolsSelector         = internationalSymbolsSelector,
		ornamentSetsMathSymbolsSelector         = mathSymbolsSelector,
	
	glyphAlternativesType = characterAlternativesType
	
};


#ifdef __cplusplus
}
#endif

#endif /* __LAYOUTFEATURECONSTANTS__ */
