
/*
	File:		OffscreenLibrary.h
	
	Contains:	offscreen support library interfaces
	
	Written By:	Cary Clark, Michael Fairman, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink
	
	Copyright:	1992-1995 by Apple Computer, Inc.  All rights reserved.
	
	Change History (most recent first):
	
		 <2>	  1/9/95	JD		changed 'boolean' to 'Boolean'
		 <1>	  1/9/95	JD		First checked in.

*/

#ifndef __OFFSCREENLIBRARY__
#define __OFFSCREENLIBRARY__

#include <GXTypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/* for compatibility with old headers */
#define offscreenLibraryIncludes


struct offscreen {
	gxShape		draw;	/* a gxBitmap gxShape which, when drawn, will transfer the offscreen to the display */
	gxTransform	xform;	/* a gxTransform that will cause shapes owning it to draw offscreen. */
	gxViewDevice	device;	/* the offscreen gxViewDevice whose gxColorSpace, etc. you may change */
	gxViewPort	port;		/* the offscreen gxViewPort which may be placed in any gxTransform's gxViewPort list */
	gxViewGroup	group;	/* the global space in which the gxViewPort and gxViewDevice exist */
};
typedef struct offscreen offscreen;

typedef struct viewPortBufferRecord **viewPortBuffer;

/* (see the .c file for routine explanations) */
void CreateOffscreen(offscreen *target, gxShape bitmapShape);
void DisposeOffscreen(offscreen *target);
void CopyToBitmaps(gxShape target, gxShape source);

viewPortBuffer NewViewPortBuffer(gxViewPort originalPort);
void DisposeViewPortBuffer(viewPortBuffer target);
Boolean ValidViewPortBuffer(viewPortBuffer target);
Boolean UpdateViewPortBuffer(viewPortBuffer target);
gxViewPort GetViewPortBufferViewPort(viewPortBuffer source);
gxShape GetViewPortBufferShape(viewPortBuffer source);

#ifdef __cplusplus
}
#endif

#endif /* __OFFSCREENLIBRARY__ */
