/*
	File:			GXEdit.h
	
	Contains:
	
	Written by:	Barton R. House
	
	Copyright:	 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#ifndef _H_GXEdit_
#define	_H_GXEdit_

#ifndef __PRINTINGMANAGER__
	#include "PrintingManager.h"
#endif

#ifndef graphicsTypesIncludes
	#include "graphics types.h"
#endif

#ifndef fontTypesIncludes
	#include "font types.h"
#endif

#ifndef layoutTypesIncludes
	#include "layout types.h"
#endif

typedef enum {
	gx_edit_no_error = 0,
	gx_edit_internal_fatal_error,
	gx_edit_warning,
	gx_edit_bad_magic
} GXEditError;

#define	kGXEditNumReplace		3

typedef struct {
	
	long			startPointSize;
	long			endPointSize;
	long			stepPointSize;
	
	Boolean		replace[kGXEditNumReplace];
	Str255		replaceStr[kGXEditNumReplace];
	short		fromChar[kGXEditNumReplace];
	short		toChar[kGXEditNumReplace];
	
	Boolean		glyphIndicies;
	
} GXEditProofRec, * GXEditProofPtr;

typedef gxRunFeature gxFontRunFeature;

typedef void * GXEditDoc;
typedef void (* GXEditUserErrorProc) (GXEditError, void * reference);

#ifdef __cplusplus
extern "C" {
#endif

extern void GXEditNew(GXEditDoc * doc, gxRectangle * viewRect, gxViewPort docViewPort, Fixed leftMargin, Fixed rightMargin);
extern void GXEditDispose(GXEditDoc doc);

extern void GXEditClick(GXEditDoc doc, gxPoint where, Boolean extend);
extern void GXEditScroll(GXEditDoc doc, short dv);
extern void GXEditSize(GXEditDoc doc, gxRectangle * viewRect);
extern void GXEditKey(GXEditDoc doc, char key);
extern void GXEditClear(GXEditDoc doc);
extern void GXEditInsert(GXEditDoc doc, void * text, short numText);

extern long GXEditGetTop(GXEditDoc doc);
extern long GXEditGetMaxTop(GXEditDoc doc);
extern long GXEditHeight(GXEditDoc doc);

extern void GXEditSetTextFonts(GXEditDoc doc, short numFonts, gxFont * srcFonts, gxFont * dstFonts);
extern void GXEditGetTextFonts(GXEditDoc doc, short * numFonts, gxFont * fonts);

extern void GXEditSetTextSize(GXEditDoc doc, short size);
extern short GXEditGetTextSize(GXEditDoc doc);

extern void GXEditSetFeatures(GXEditDoc doc, gxFont fontId, short numFeatureGroups,
									short * srcFeatureGroupSizes,
									gxRunFeature ** srcFeatureGroups,
									short * dstFeatureGroupSizes,
									gxRunFeature ** dstFeatureGroups);
extern void GXEditGetFeatures(GXEditDoc doc, gxFont fontId, short * numFeatureGroupsPtr,
									short *** featureGroupSizesPtr,
									gxRunFeature **** featureGroupsPtr);

extern void GXEditGetVariations(GXEditDoc doc, gxFont fontId, short * numInstancesPtr,
									short *** instanceSizesPtr, gxFontVariation **** instancesPtr);
extern void GXEditSetVariations(GXEditDoc doc, gxFont fontId, short numVariations,
									gxFontVariation * variations);
									
extern void GXEditGetSelectionStyles(GXEditDoc doc, short * numStyles, gxStyle * styles);
extern void GXEditSetSelectionStyles(GXEditDoc doc, gxStyle * toStyles);

extern void GXEditSave(GXEditDoc doc, short fRefNum);
extern void GXEditOpen(GXEditDoc * doc, Rect * viewRect, gxViewPort docViewPort, short fRefNum,
									Fixed leftMargin, Fixed rightMargin);

extern void GXEditPrint(GXEditDoc doc, gxJob printJob);

extern void GXEditCut(GXEditDoc doc);
extern void GXEditCopy(GXEditDoc doc);
extern void GXEditPaste(GXEditDoc doc);

extern long GXEditGetScrapLength(GXEditDoc doc);
extern long GXEditGetTextLength(GXEditDoc doc);
extern long GXEditGetSelectionLength(GXEditDoc doc);
extern void GXEditSetSelection(GXEditDoc doc, long selStart, long selEnd);

extern void GXEditGetRunControls(GXEditDoc doc, gxRunControls * runControls, gxRunControls * mask);
extern void GXEditSetRunControls(GXEditDoc doc, gxRunControls * runControls, gxRunControls * mask);

extern void GXEditGetFontRunControls(GXEditDoc doc, gxFont fontID, gxRunControls * runControls, gxRunControls * mask);

extern void GXEditGetLayoutOptions(GXEditDoc doc, gxLayoutOptions * options, gxLayoutOptions * mask);
extern void GXEditSetLayoutOptions(GXEditDoc doc, gxLayoutOptions * options, gxLayoutOptions * mask);

extern Boolean GXEditIdle(GXEditDoc doc);
extern void GXEditUpdate(GXEditDoc doc, gxRectangle * updateRect);

extern void GXEditGetError(GXEditDoc doc, GXEditError * error, GXEditError * stickyError);
extern void GXEditGetUserError(GXEditDoc doc, GXEditUserErrorProc * userProc, void ** reference);
extern void GXEditSetUserError(GXEditDoc doc, GXEditUserErrorProc userProc, void * reference);

extern void GXEditFlushCaches(GXEditDoc doc);
extern void GXEditProof(GXEditDoc doc, GXEditProofPtr proof);

extern void GXEditActivate(GXEditDoc doc, Boolean activate);

extern void GXEditSetEncoding(GXEditDoc doc, gxFontPlatform platform, gxFontScript script, gxFontLanguage language);

extern void GXEditSetVerticalText(GXEditDoc doc, Boolean enable);
extern void GXEditGetVerticalText(GXEditDoc doc, Boolean * enable);

extern void GXEditGetSelectionEncoding(GXEditDoc doc, gxFontPlatform * platform, gxFontScript * script, gxFontLanguage * language);
extern void GXEditSetSelectionEncoding(GXEditDoc doc, gxFontPlatform platform, gxFontScript script, gxFontLanguage language);

#ifdef __cplusplus
}
#endif

#endif


