/*
	File:			GXEditError.c
	
	Contains:
	
	Written by:		Barton R. House
	
	Copyright:		 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#include <Traps.h>

#include "GXEdit.h"
#include "GXEditDoc.h"
#include "GXEditError.h"

void gxEditPostError(DocPtr dp, GXEditError error)
{
	switch(error) {
	case gx_edit_no_error:	/* ??? */ break;
	case gx_edit_internal_fatal_error: DebugStr("\pGXEdit fatal error"); break;
	case gx_edit_warning: DebugStr("\pGXEdit warning"); break;
	case gx_edit_bad_magic: DebugStr("\pGXEdit bad magic"); break;
	}
		
	if(dp != nil) {
	
		if(dp->errorProc != nil)
			(*dp->errorProc)(error, dp->errorProcRef);
		
		if(dp->stickyError == gx_edit_no_error)
			dp->stickyError = error;
			
		dp->error = error;
		
	}
	
}

void gxEditSetUserError(DocPtr dp, GXEditUserErrorProc errorProc, void * reference)
{
	dp->errorProc = errorProc;
	dp->errorProcRef = reference;
}

void gxEditGetUserError(DocPtr dp, GXEditUserErrorProc * errorProc, void ** reference)
{
	if(errorProc != nil)
		*errorProc = dp->errorProc;
		
	if(reference != nil)
		*reference = dp->errorProcRef;
}

void gxEditGetError(DocPtr dp, GXEditError * error, GXEditError * stickyError)
{
	
	if(error != nil)
		*error = dp->error;
	
	if(stickyError != nil)
		*stickyError = dp->stickyError;
		
	dp->stickyError = gx_edit_no_error;
	
}
