/*
	Spinning Colored Ovals
	
	The application will create an oval the slightly smaller then the size of the window.  The oval will be framed and 
	filled with a color from HsvSpace.  The frame color will be the opposite of the fill color.  The oval is created by 
	using the the NewOval routine from the "graphics library".
	
	NOTES:
	 This file requires the following files to run correctly:
	    "graphics shell.c", "ColorLibrary.c", "GraphicsDebugLibrary.c", "OvalLibrary.c", 
	    "ShapeLibrary.c", "TransformLibrary.c".

	 For the best printing results, print this file in "landscape".
	
	
	Change History:

	   4/96	bob		Updated #includes to support changed GX Library names.
					Changed fixed to Fixed.
					Updated the note regarding the files needed to run.
					Updated the copyright date.

	1990 - 1996  Apple Computer, Inc.
	All rights reserved.
*/

#include <Events.h>
#include <Windows.h>

#include "GraphicsLibraries.h"
#include <GXEnvironment.h>
#include "graphics shell.h"

#define f(a,b)	(((Fixed) (a) << 16) + (b))
#define kNumberOfOvalsDrawn 	90

//
//  Set up the title and size of the window 
//
Str255 			gWindowTitle = "\p Spinning Colored Ovals";
Rect 			gWindowQDRect  = {50, 50, 425, 525};

//
//	gGraphicsHeapSize sets the size of the graphics gxHeap created by calling the GXNewGraphicsClient routine
//	in main () within graphics shell.c.  You can determine the amount of graphics gxHeap required by using GraphicsBug.
//	With  gGraphicsHeapSize set to 20k, I had 3 free blocks left in the graphics gxHeap. Sounds good to me.
//
long			gGraphicsHeapSize = 20;


gxShape 		gOvalShape;
gxColor 		gOutsideOvalColor, gInsideOvalColor;
short			gTotalnumberOfOvalsDrawn;
gxRectangle 	gFixedWindowBounds;	 			/**  bounding box of the window in local coordinates  **/



/*------ DoInitialization ---------------------------------------------------------------------------------*/

void DoInitialization(aWindow)
WindowPtr aWindow;
{
	/**   Get the local bounding gxShape of the window  **/
	GXGetShapeBounds(gWindowBoundsShape, 0L, &gFixedWindowBounds);
	
	/**  Inset the size of the oval relative to the window's bounding gxShape   **/
	gFixedWindowBounds.right -= ff(40);
	gFixedWindowBounds.bottom -= ff(190); 
	
	gOvalShape = NewOval(&gFixedWindowBounds);		
	
	/**  Set up the hsv colorspace for the inside & outside oval...  **/
	gInsideOvalColor.space = gxHSVSpace;
	gInsideOvalColor.profile = nil;
	gInsideOvalColor.element.hsv.hue = 0x0000;
	gInsideOvalColor.element.hsv.saturation = 0xFFFF;
	gInsideOvalColor.element.hsv.value = 0xFC00; 
	
	gOutsideOvalColor.space = gxHSVSpace;
	gOutsideOvalColor.profile = nil;
	gOutsideOvalColor.element.hsv.hue = 0x0000;
	gOutsideOvalColor.element.hsv.saturation = 0xFFFF;
	gOutsideOvalColor.element.hsv.value = 0x7FFF;  

	GXSetShapePen(gOvalShape, ff(2));

	gTotalnumberOfOvalsDrawn = 0;

	/**   Move the oval into the imaging area of the window  **/
	GXMoveShapeTo (gOvalShape, ff(20), ff(95));
}


/*------ DoDraw ---------------------------------------------------------------------------------------*/

void DoDraw(aWindow)
WindowPtr aWindow;
{
	gxRectangle 	ovalShapeBounds;
	Fixed 			x, y; 
	
	/**  Create a new oval after we have drawn a bunch of ovals  **/
	if (gTotalnumberOfOvalsDrawn == kNumberOfOvalsDrawn) {
	
	  	GXDisposeShape(gOvalShape);  
		gOvalShape = NewOval(&gFixedWindowBounds);		
		GXSetShapePen(gOvalShape, ff(2));

		/**   Move the oval into the imaging area of the window  **/
		GXMoveShapeTo (gOvalShape, ff(20), ff(95)); 

	  	gTotalnumberOfOvalsDrawn = 0;
	}
	
	/**  Set the gxColor and frame for the outside oval  **/
	GXSetShapeColor(gOvalShape, &gOutsideOvalColor);
	GXSetShapeFill (gOvalShape, gxClosedFrameFill);
	GXDrawShape (gOvalShape);

	/**  Set the gxColor and frame for the inside oval  **/
 	GXSetShapeColor(gOvalShape, &gInsideOvalColor);
	GXSetShapeFill (gOvalShape,gxSolidFill);
	GXDrawShape(gOvalShape);

	/**  Change the gxColor of both ovals  **/
	gInsideOvalColor.element.hsv.hue += 0x0600;
	gOutsideOvalColor.element.hsv.hue += 0x0600;

	/**   
		Get the bounds of the oval each time through DoDraw. Why? We are scaling the oval with 
		a call to GXScaleShape each time through DoDraw, which changes the bounding box of the oval.
	**/
	GXGetShapeBounds(gOvalShape, 0L, &ovalShapeBounds);
	x = ovalShapeBounds.left + ovalShapeBounds.right >> 1;
	y = ovalShapeBounds.top + ovalShapeBounds.bottom >> 1;
			
	GXScaleShape(gOvalShape, f(0, 0xF800), f(0, 0xF800), x, y);
	GXRotateShape(gOvalShape, ff(25), x, y);

	gTotalnumberOfOvalsDrawn++;
}


/*------ DoDispose -------------------------------------------------------------------------------------*/

void DoDispose(aWindow)
WindowPtr aWindow;
{
	//  
	//	You should always dispose of your GX graphics objects before tossing your window. Why? It's generally good 
	//	form and this approach guarantees that everything is disposed. If you had not disposed of everything, the
	//	call to DisposeWindow should dispose of the objects. If you are running the debugging version of the 
	//	SecretGraphics init with notices set, you will receive a notice that you had not disposed of everything. You
	//	can turn notices on in this file by setting gDebugging = TRUE (above).
	//  
   	DisposeWindow(aWindow);
	GXDisposeShape(gOvalShape); 
	GXDisposeShape(gWindowBoundsShape);  
}
	


/*------ DoClick ---------------------------------------------------------------------------------------*/

void DoClick( orgMouseLoc, theWindow )
gxPoint		orgMouseLoc;
WindowPtr 	theWindow;
{
}


/*------ DoIdle ----------------------------------------------------------------------------------------*/

void DoIdle(aWindow)
WindowPtr aWindow;
{
	DoDraw(aWindow);
}
