/*
	FILENAME
		NewApp.r

	DESCRIPTION
		Contains standard imaging and Finder resource descriptions for our
		driver.

	COPYRIGHT
		Copyright  1995 Apple Computer, Inc.
		All rights reserved.
	
	Modification history
		10/04/95 - David Hayward -	Version 1.0.4 modified code so that
									the driver can be build under MPW,
									Metrowerks, and Symantec.  In general,
									all that was required to do this was 
									to add an inline-assembly jumptable
									and to store all globals off of the
									message manager instance context.
									Also made a few changes so that the
									driver can be rebuilt to support any
									resolution by changing the #defines
									kResolution and kPatStretch in
									"CommonDefines.h"

		06/14/95 - Dave Hersey -	Version 1.0.3 to fix a bug in
									CustomBufferingAndIO.c when creating
									high-res PICTs, and to make the size
									of buffers more flexible.

		05/26/95 - Dave Hersey -	Version 1.0.2 to add the new 'outp'
									desktop printer resource in NewApp.c.

		05/03/95 - Dave Hersey -	Version 1.0.1 to fix some minor bugs in
									CustomBufferingAndIO.c.

		01/14/95 - Dave Hersey -	Created from the shell of a hollowed-out
									ImageWriter driver.
*/

#include "Types.r"
#include "SysTypes.r"
#include "GXPrintingResTypes.r"
#include "GXPrintingResEquates.r"
#include "CommonDefines.h"


// Version resource required by all GX printer drivers.

resource 'vers' (gxPrintingDriverBaseID, purgeable) {
	0x01,
	0x00,
	release,
	0x00,
	verUS,
	"1.00",
	"1.00, Copyright \251 Apple Computer, Inc. 1995"
};


// Normal 'vers' resources so we can tell what version we've got.

resource 'vers' (1, sysheap, purgeable) {
	0x1,
	0x0,
	release,
	0x4,
	verUS,
	"1.0.4",
	"1.0.4,  Apple Computer, Inc. 1995"
};

resource 'vers' (2, sysheap, purgeable) {
	0x1,
	0x0,
	release,
	0x4,
	verUS,
	"1.0.4",
	"Sample Printer Driver"
};


/* ------------------------------------------------------------------------ */
/* RESOURCE FOR TELLING THE SYSTEM WHAT KIND OF DRIVER WE ARE				*/
/* ------------------------------------------------------------------------ */

// select which imaging system to use and the ID of the imaging override resource, 
resource gxImagingSystemSelectorType (gxImagingSystemSelectorID, sysHeap)
{
	gxRasterPrinterType	// we use the raster system
};


/* ------------------------------------------------------------------------ */
/* RESOURCES FOR TELLING THE SYSTEM WHAT MESSAGES WE'RE OVERRIDING			*/
/* ------------------------------------------------------------------------ */

// Overrides for raster specific messages, and where to find them in the jump table
// This resource must be kept in sync with the assembly
// jump table in NewApp.c and NewApp.a
resource gxOverrideType (gxDriverImagingOverrideID, sysheap, purgeable)
{
	{
		gxRasterDataIn,				NewSegID, 4
	};
};

// Overrides for universal messages, and where to find them in the jump table
// This resource must be kept in sync with the assembly
// jump table in NewApp.c and NewApp.a
resource gxOverrideType (gxDriverUniversalOverrideID, sysheap, purgeable)
{
	{
		gxImageJob,					NewSegID, 8,
		gxStartSendPage,			NewSegID, 12,
		gxFinishSendPage,			NewSegID, 16,
		gxSetupImageData,			NewSegID, 20,
		gxOpenConnection,			NewSegID, 24,
		gxCloseConnection,			NewSegID, 28,
		gxCleanupOpenConnection,	NewSegID, 32,
		gxBufferData,				NewSegID, 36,
		gxFreeBuffer,				NewSegID, 40,
		gxWriteData,				NewSegID, 44,
		gxDumpBuffer,				NewSegID, 48,
		gxDefaultDesktopPrinter,	NewSegID, 52
	};
};

// Status message resource
resource 'stat' (kTransmissionStatID, sysheap, purgeable)
{
	DriverCreator,
	{
		informationalStatus,	1,	0,	"Writing part of the page";
		informationalStatus,	2,	0,	"Preparing part of the page";
		pageTransmission,		3,	0,	"Saving PICT data";
	}
};


/* ------------------------------------------------------------------------ */
/* I/O PREFERENCES															*/
/* ------------------------------------------------------------------------ */

resource gxUniversalIOPrefsType (gxUniversalIOPrefsID, sysheap, purgeable)
{
	customIO,
	0,
	0,
	0,
	0,
	0
};


/* ------------------------------------------------------------------------ */
/* RASTER DRIVER DATA PREFERENCES											*/
/* ------------------------------------------------------------------------ */
// This resource customizes the behavior of the raster generic driver, allowing
// us to specify our imaging options, as well as format/page scaling.
// All raster drivers must have this resource.

resource gxRasterPrefsType (gxRasterPrefsID, sysheap, purgeable)
{
	9,							// gxDontResolveTransferModes +gxSendAllBands
	kResolution*65536,			// device horizontal resolution
	kResolution*65536,			// device vertical resolution
	1,							// min band size == 1 pixel high
	0,							// max band size (0 is full page)
	0x00008000,					// RAM percentage (50%)
	100*1024,					// RAM slop (100K)
	32,							// 32 bit device. 8<unused> 8R, 8G,8B
	{
		gxDontSetHalftone,									// No halftoning, please.
		0,													// UNUSED -- Halftone angle.
		0,													// UNUSED -- Halftone frequency.
		0,													// UNUSED -- Halftone dot type.
		0,													// UNUSED -- Tint type.
		0,0,0,0,0,0, 										// UNUSED -- dot color.
		0,0,0,0,0,0, 										// UNUSED -- Background color.
		0,													// UNUSED -- Tint color space.
		gxRGB32Space,										// RGB 32-bit color space
		gxNoSet,											// No color set
		gxNoProfile											// No profile specified
	};
};


/* ------------------------------------------------------------------------ */
/* RESOURCES FOR THE BNDL AND ICON											*/
/* ------------------------------------------------------------------------ */

type DriverCreator
{
	pstring;
};

resource 'BNDL' (gxPrintingDriverBaseID + 1, sysheap, purgeable) {
	DriverCreator,
	0,
	{
		'ICN#', { 0, gxPrintingDriverBaseID + 2; 1, gxPrintingDriverBaseID + 3; 2, gxPrintingDriverBaseID + 4; 3, gxPrintingDriverBaseID + 5; 4, gxPrintingDriverBaseID + 6; 5, gxPrintingDriverBaseID + 7; 6, gxPrintingDriverBaseID + 8},
		'FREF', { 0, gxPrintingDriverBaseID + 2; 1, gxPrintingDriverBaseID + 3; 2, gxPrintingDriverBaseID + 4; 3, gxPrintingDriverBaseID + 5; 4, gxPrintingDriverBaseID + 6; 5, gxPrintingDriverBaseID + 7; 6, gxPrintingDriverBaseID + 8; 0, gxPrintingDriverBaseID + 1}
	}
};

resource 'FREF' (gxPrintingDriverBaseID + 1, sysheap, purgeable) { DriverType, 0, "" };
resource 'FREF' (gxPrintingDriverBaseID + 2, sysheap, purgeable) { 'dpnn', 0, "" };
resource 'FREF' (gxPrintingDriverBaseID + 3, sysheap, purgeable) { 'dpns', 1, "" };
resource 'FREF' (gxPrintingDriverBaseID + 4, sysheap, purgeable) { 'dpcn', 2, "" };
resource 'FREF' (gxPrintingDriverBaseID + 5, sysheap, purgeable) { 'dpcs', 3, "" };
resource 'FREF' (gxPrintingDriverBaseID + 6, sysheap, purgeable) { 'dpna', 4, "" };
resource 'FREF' (gxPrintingDriverBaseID + 7, sysheap, purgeable) { 'dvcf', 4, "" };
resource 'FREF' (gxPrintingDriverBaseID + 8, sysheap, purgeable) { 'dppz', 5, "" };


/* Driver icons */

// non-current, non-shared
resource 'ics#' (gxPrintingDriverBaseID + 2, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"3FE0 2030 2028 203C 2F34 294C 2F4C 2034 E107 A285 E107 FFFF"
		$"8001 FFFF 2004 3FFC",
		/* [2] */
		$"3FE0 3FF0 3FF8 3FFC 3FFC 3FFC 3FFC 3FFC FFFF FFFF FFFF FFFF"
		$"FFFF FFFF 3FFC 3FFC"
	}
};

resource 'ics4' (gxPrintingDriverBaseID + 2, sysheap, purgeable) {
	$"00FF FFFF FFF0 0000 00FC 0C0C 0CFF 0000 00F0 C0C0 C0FC F000"
	$"00FC 0C0C 0CFF FF00 00F0 FFFF C0FF CF00 00FC F87F 0F44 FF00"
	$"00F0 FFFF CF44 FF00 00FC 0C0C 0CFF 0F00 FFF0 C0CF C0C0 CFFF"
	$"FCFC 0CF5 FC0C 0FCF FFF0 C0CF C0C0 CFFF FFFF FFFF FFFF FFFF"
	$"FCCC CCCC CCCC CCCF FFFF FFFF FFFF FFFF 00F0 C0C0 C0C0 CF00"
	$"00FF FFFF FFFF FF"
};

resource 'ics8' (gxPrintingDriverBaseID + 2, sysheap, purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FF00 0000 0000 0000 FFF5 F5F5 F5F5"
	$"F5F5 FFFF 0000 0000 0000 FFF5 F5F5 F5F5 F5F5 FF2B FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 FFFF FFFF 0000 0000 FFF5 FFFF FFFF"
	$"F5F5 FFFF F5FF 0000 0000 FFF5 FF48 48FF F5FF 1313 FFFF 0000"
	$"0000 FFF5 FFFF FFFF F5FF 1313 FFFF 0000 0000 FFF5 F5F5 F5F5"
	$"F5F5 FFFF F5FF 0000 FFFF FFF5 F5F5 F5FF F5F5 F5F5 F5FF FFFF"
	$"FFF8 FFF5 F5F5 FF7F FFF5 F5F5 F5FF F8FF FFFF FFF5 F5F5 F5FF"
	$"F5F5 F5F5 F5FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFF8 F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8FF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ICN#' (gxPrintingDriverBaseID + 2, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0FFF FE00 0800 0300 0800 0280 0800 0240 0800 0220 0800 0210"
		$"0800 03F8 08FF 0008 0881 0E08 0881 1108 0881 2088 08FF 2088"
		$"0800 2088 3801 110E 5802 8E0D 4804 4009 4808 2009 6810 100B"
		$"5808 200D 3804 400E 3FFF FFFE 5555 5555 4000 0001 4000 0001"
		$"6AAA AAAB 5555 5555 3FFF FFFE 0FFF FFF8 0800 0008 0800 0008"
		$"0FFF FFF8",
		/* [2] */
		$"0FFF FE00 0FFF FF00 0FFF FF80 0FFF FFC0 0FFF FFE0 0FFF FFF0"
		$"0FFF FFF8 0FFF FFF8 0FFF FFF8 0FFF FFF8 0FFF FFF8 0FFF FFF8"
		$"0FFF FFF8 3FFF FFFE 7FFF FFFF 7FFF FFFF 7FFF FFFF 7FFF FFFF"
		$"7FFF FFFF 3FFF FFFE 3FFF FFFE 7FFF FFFF 7FFF FFFF 7FFF FFFF"
		$"7FFF FFFF 7FFF FFFF 3FFF FFFE 0FFF FFF8 0FFF FFF8 0FFF FFF8"
		$"0FFF FFF8"
	}
};

resource 'icl4' (gxPrintingDriverBaseID + 2, sysheap, purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FFF0 0000 0000 0000 F0C0 C0C0 C0C0"
	$"C0C0 C0FF 0000 0000 0000 FC0C 0C0C 0C0C 0C0C 0CFC F000 0000"
	$"0000 F0C0 C0C0 C0C0 C0C0 C0FC CF00 0000 0000 FC0C 0C0C 0C0C"
	$"0C0C 0CFC CCF0 0000 0000 F0C0 C0C0 C0C0 C0C0 C0FC CCCF 0000"
	$"0000 FC0C 0C0C 0C0C 0C0C 0CFF FFFF F000 0000 F0C0 FFFF FFFF"
	$"C0C0 C0C0 C0C0 F000 0000 FC0C F778 777F 0C0C FFFC 0C0C F000"
	$"0000 F0C0 F787 778F C0CF 444F C0C0 F000 0000 FC0C F877 787F"
	$"0CF4 4444 FC0C F000 0000 F0C0 FFFF FFFF C0F4 4444 F0C0 F000"
	$"0000 FC0C 0C0C 0C0C 0CF4 4444 FC0C F000 00FF F0C0 C0C0 C0CF"
	$"C0CF 444F C0C0 FFF0 0FDD FC0C 0C0C 0CF5 FC0C FFFC 0C0C FEDF"
	$"0F00 F0C0 C0C0 CF55 5FC0 C0C0 C0C0 FE0F 0FCC FC0C 0C0C F555"
	$"55FC 0C0C 0C0C FECF 0FDD F0C0 C0CF 5555 555F C0C0 C0C0 FEDF"
	$"0FEE FC0C 0C0C F555 55FC 0C0C 0C0C FEEF 00FF F0C0 C0C0 CF55"
	$"5FC0 C0C0 C0C0 FFF0 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"0FDD DDDD DDDD DDDD DDDD DDDD DDDD DDDF 0F00 0000 0000 0000"
	$"0000 0000 0000 000F 0FCC CCCC CCCC CCCC CCCC CCCC CCCC CCCF"
	$"0FDD DDDD DDDD DDDD DDDD DDDD DDDD DDDF 0FEE EEEE EEEE EEEE"
	$"EEEE EEEE EEEE EEEF 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"0000 FEEE EEEE EEEE EEEE EEEE EEEE F000 0000 F0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 F000 0000 FC0C 0C0C 0C0C 0C0C 0C0C 0C0C F000"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF F0"
};

resource 'icl8' (gxPrintingDriverBaseID + 2, sysheap, purgeable) {
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 0000 0000 0000 0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 FFFF 0000 0000 0000 0000 0000 0000 FFF5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B FF00 0000 0000 0000"
	$"0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B"
	$"2BFF 0000 0000 0000 0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 FF2B 2B2B FF00 0000 0000 0000 0000 FFF5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B 2B2B 2BFF 0000 0000"
	$"0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FFFF"
	$"FFFF FFFF FF00 0000 0000 0000 FFF5 F5F5 FFFF FFFF FFFF FFFF"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000 0000 0000 FFF5 F5F5"
	$"FF48 4848 4848 48FF F5F5 F5F5 FFFF FFF5 F5F5 F5F5 FF00 0000"
	$"0000 0000 FFF5 F5F5 FF48 4848 4848 48FF F5F5 F5FF 1313 13FF"
	$"F5F5 F5F5 FF00 0000 0000 0000 FFF5 F5F5 FF48 4848 4848 48FF"
	$"F5F5 FF13 1313 1313 FFF5 F5F5 FF00 0000 0000 0000 FFF5 F5F5"
	$"FFFF FFFF FFFF FFFF F5F5 FF13 1313 1313 FFF5 F5F5 FF00 0000"
	$"0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF13 1313 1313"
	$"FFF5 F5F5 FF00 0000 0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5FF"
	$"F5F5 F5FF 1313 13FF F5F5 F5F5 FFFF FF00 00FF F9F9 FFF5 F5F5"
	$"F5F5 F5F5 F5F5 FF7F FFF5 F5F5 FFFF FFF5 F5F5 F5F5 FFF9 F9FF"
	$"00FF F5F5 FFF5 F5F5 F5F5 F5F5 F5FF 7F7F 7FFF F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFF9 F5FF 00FF F7F7 FFF5 F5F5 F5F5 F5F5 FF7F 7F7F"
	$"7F7F FFF5 F5F5 F5F5 F5F5 F5F5 FFF9 F7FF 00FF F9F9 FFF5 F5F5"
	$"F5F5 F5FF 7F7F 7F7F 7F7F 7FFF F5F5 F5F5 F5F5 F5F5 FFF9 F9FF"
	$"00FF FCFC FFF5 F5F5 F5F5 F5F5 FF7F 7F7F 7F7F FFF5 F5F5 F5F5"
	$"F5F5 F5F5 FFFC FCFF 0000 FFFF FFF5 F5F5 F5F5 F5F5 F5FF 7F7F"
	$"7FFF F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FF00 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"00FF F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 F9F9 F9FF 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 00FF F7F7 F7F7 F7F7"
	$"F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7FF"
	$"00FF F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 F9F9 F9FF 00FF FCFC FCFC FCFC FCFC FCFC FCFC FCFC"
	$"FCFC FCFC FCFC FCFC FCFC FCFC FCFC FCFF 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 0000 FFFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB"
	$"FBFB FBFB FF00 0000 0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000 0000 0000 FFF5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000"
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FF"
};

// non-current, shared
resource 'ICN#' (gxPrintingDriverBaseID + 3, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0000 0000 01FF FC00 0100 0600 0100 0500 0100 0480 010F 8440"
		$"0108 9C20 0108 A7F0 070F A21C 0900 2212 0900 9C12 0901 4012"
		$"0F02 201E 07FF FFFC 0800 0002 0800 0002 0800 0002 0FFF FFFE"
		$"07FF FFFC 01FF FFF0 0100 0150 01FF FF70 0000 0140 0000 03E0"
		$"0000 0220 0000 0220 0000 03E0 0000 02A0 BFFF FC9D 0000 0140"
		$"BFFF FE3D",
		/* [2] */
		$"0000 0000 01FF FC00 01FF FE00 01FF FF00 01FF FF80 01FF FFC0"
		$"01FF FFE0 01FF FFF0 07FF FFFC 0FFF FFFE 0FFF FFFE 0FFF FFFE"
		$"0FFF FFFE 07FF FFFC 0FFF FFFE 0FFF FFFE 0FFF FFFE 0FFF FFFE"
		$"07FF FFFC 01FF FFF0 01FF FFF0 01FF FFF0 0000 01C0 0000 03E0"
		$"0000 03E0 0000 03E0 0000 03E0 0000 03E0 FFFF FFFF FFFF FF7F"
		$"FFFF FE3F"
	}
};

resource 'icl4' (gxPrintingDriverBaseID + 3, sysheap, purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 000F FFFF FFFF"
	$"FFFF FF00 0000 0000 0000 000F 0000 0000 0000 0FF0 0000 0000"
	$"0000 000F 0000 0000 0000 0FCF 0000 0000 0000 000F 0000 0000"
	$"0000 0FCC F000 0000 0000 000F 0000 FFFF F000 0FCC CF00 0000"
	$"0000 000F 0000 F877 F00F FFCC CCF0 0000 0000 000F 0000 F777"
	$"F0F4 4FFF FFFF 0000 0000 0FFF 0000 FFFF F0F4 44F0 000F FF00"
	$"0000 FDDF 0000 0000 00F4 44F0 000F DDF0 0000 F00F 0000 0000"
	$"F00F FF00 000F D0F0 0000 FDDF 0000 000F 5F00 0000 000F DDF0"
	$"0000 FEEF 0000 00F5 55F0 0000 000F EEF0 0000 0FFF FFFF FFFF"
	$"FFFF FFFF FFFF FF00 0000 FDDD DDDD DDDD DDDD DDDD DDDD DDF0"
	$"0000 F000 0000 0000 0000 0000 0000 00F0 0000 FCCC CCCC CCCC"
	$"CCCC CCCC CCCC CCF0 0000 FDDD DDDD DDDD DDDD DDDD DDDD DDF0"
	$"0000 0FFF FFFF FFFF FFFF FFFF FFFF FF00 0000 000F EEEE EEEE"
	$"EEEE EEEE EEEF 0000 0000 000F 0000 0000 0000 000D CD0F 0000"
	$"0000 000F FFFF FFFF FFFF FFFE CEFF 0000 0000 0000 0000 0000"
	$"0000 000E CE00 0000 0000 0000 0000 0000 0000 00FF FFF0 0000"
	$"0000 0000 0000 0000 0000 00FC CCF0 0000 0000 0000 0000 0000"
	$"0000 00FC CCF0 0000 0000 0000 0000 0000 0000 00FF FFF0 0000"
	$"0000 0000 0000 0000 0000 00FC FCE0 0000 DDDD DDDD DDDD DDDD"
	$"EEEE EECC FCCE EEED CCCC CCCC CCCC CCCC CCCC CCCF 0FCC CCCC"
	$"DDDD DDDD DDDE EEEE EEEE FFF0 00FF EEEE"
};

resource 'icl8' (gxPrintingDriverBaseID + 3, sysheap, purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 00FF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF 0000 0000 0000 0000 0000 0000 0000 0000 00FF"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF FF00 0000 0000 0000 0000"
	$"0000 0000 0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 2BFF"
	$"0000 0000 0000 0000 0000 0000 0000 00FF F5F5 F5F5 F5F5 F500"
	$"00F5 F5F5 F5FF 2B2B FF00 0000 0000 0000 0000 0000 0000 00FF"
	$"F5F5 F5F5 FFFF FFFF FFF5 F5F5 F5FF 2B2B 2BFF 0000 0000 0000"
	$"0000 0000 0000 00FF F5F5 F5F5 FF48 4848 FFF5 F5FF FFFF 2B2B"
	$"2B2B FF00 0000 0000 0000 0000 0000 00FF F5F5 F5F5 FF48 4848"
	$"FFF5 FF13 13FF FFFF FFFF FFFF 0000 0000 0000 0000 00FF FFFF"
	$"F5F5 F5F5 FFFF FFFF FFF5 FF13 1313 FFF5 F5F5 F5FF FFFF 0000"
	$"0000 0000 FFF9 F9FF F5F5 F5F5 F5F5 F5F5 F5F5 FF13 1313 FFF5"
	$"F5F5 F5FF F9F9 FF00 0000 0000 FFF5 F5FF F5F5 F5F5 F5F5 F5F5"
	$"FFF5 F5FF FFFF F5F5 F5F5 F5FF F9F5 FF00 0000 0000 FFF9 F9FF"
	$"F5F5 F5F5 F5F5 F5FF 7FFF F5F5 F5F5 F5F5 F5F5 F5FF F9F9 FF00"
	$"0000 0000 FFFC FCFF F5F5 F5F5 F5F5 FF7F 7F7F FFF5 F5F5 F5F5"
	$"F5F5 F5FF FCFC FF00 0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000 0000 0000 FFF9 F9F9"
	$"F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 FF00"
	$"0000 0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 FF00 0000 0000 FFF8 F8F8 F8F8 F8F8 F8F8 F8F8"
	$"F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 FF00 0000 0000 FFFA FAFA"
	$"FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FF00"
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF 0000 0000 0000 0000 00FF FBFB FBFB FBFB FBFB"
	$"FBFB FBFB FBFB FBFB FBFB FBFF 0000 0000 0000 0000 0000 00FF"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FA 2BFA F5FF 0000 0000"
	$"0000 0000 0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFC"
	$"2BFC FFFF 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 00FC 2BFD 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 FFFF FFFF FF00 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 FF2B"
	$"2B2B FF00 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 FF2B 2B2B FF00 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 FFFF FFFF FF00 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 FF2B"
	$"FE2B FD00 0000 0000 F6F6 F6F6 F7F7 F7F8 F8F8 F9F9 F9FA FAFA"
	$"FBFB FBFC FCFC 2B2B FE2B 2BFD FDFC FCFA F5F5 F5F5 F5F5 F6F6"
	$"F6F6 F6F6 F62B 2B2B 2B2B 2B2B 2B2B 2BFE 00FE 2B2B 2B2B 2B2B"
	$"F7F7 F8F8 F8F9 F9F9 FAFA FAFB FBFB FCFC FCFD FDFD FEFE FE00"
	$"0000 FEFE FDFD FCFC"
};

// current, non-shared
resource 'ICN#' (gxPrintingDriverBaseID + 4, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"3FFF FF80 3FFF FFC0 3FFF FFE0 3800 0270 3800 0238 3800 021C"
		$"3800 03FE 38FF 000E 3881 0E0E 3881 110E 3881 208E 38FF 208E"
		$"3800 208E 3801 110E 6802 8E0E E804 400F E808 200F E810 100F"
		$"F808 200E 7804 400E 7FFF FFFE 6000 0006 E000 0007 E000 0007"
		$"E000 0007 7FFF FFFF 3FFF FFFE 3FFF FFFE 3800 000E 3FFF FFFE"
		$"3FFF FFFE 3FFF FFFE",
		/* [2] */
		$"3FFF FF80 3FFF FFC0 3FFF FFE0 3FFF FFF0 3FFF FFF8 3FFF FFFC"
		$"3FFF FFFE 3FFF FFFE 3FFF FFFE 3FFF FFFE 3FFF FFFE 3FFF FFFE"
		$"3FFF FFFE 3FFF FFFE 7FFF FFFE FFFF FFFF FFFF FFFF FFFF FFFF"
		$"FFFF FFFE 7FFF FFFE 7FFF FFFE 7FFF FFFE FFFF FFFF FFFF FFFF"
		$"FFFF FFFF 7FFF FFFF 3FFF FFFE 3FFF FFFE 3FFF FFFE 3FFF FFFE"
		$"3FFF FFFE 3FFF FFFE"
	}
};

resource 'icl4' (gxPrintingDriverBaseID + 4, sysheap, purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF F000 0000 00FF FFFF FFFF FFFF"
	$"FFFF FFFF FF00 0000 00FF FFFF FFFF FFFF FFFF FFFF FFF0 0000"
	$"00FF F000 0000 0000 0000 00FC CFFF 0000 00FF F000 0000 0000"
	$"0000 00FC CCFF F000 00FF F000 0000 0000 0000 00FC CCCF FF00"
	$"00FF F000 0000 0000 0000 00FF FFFF FFF0 00FF F000 FFFF FFFF"
	$"0000 0000 0000 FFF0 00FF F000 F778 777F 0000 FFF0 0000 FFF0"
	$"00FF F000 F787 778F 000F 444F 0000 FFF0 00FF F000 F877 787F"
	$"00F4 4444 F000 FFF0 00FF F000 FFFF FFFF 00F4 4444 F000 FFF0"
	$"00FF F000 0000 0000 00F4 4444 F000 FFF0 00FF F000 0000 000F"
	$"000F 444F 0000 FFF0 0FFD F000 0000 00F5 F000 FFF0 0000 FFF0"
	$"FFF0 F000 0000 0F55 5F00 0000 0000 FFFF FFFC F000 0000 F555"
	$"55F0 0000 0000 FFFF FFFD F000 000F 5555 555F 0000 0000 FFFF"
	$"FFFE F000 0000 F555 55F0 0000 0000 FFF0 0FFF F000 0000 0F55"
	$"5F00 0000 0000 FFF0 0FFF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"0FFD DDDD DDDD DDDD DDDD DDDD DDDD DFF0 FFF0 0000 0000 0000"
	$"0000 0000 0000 0FFF FFFC CCCC CCCC CCCC CCCC CCCC CCCC CFFF"
	$"FFFD DDDD DDDD DDDD DDDD DDDD DDDD DFFF 0FFE EEEE EEEE EEEE"
	$"EEEE EEEE EEEE EFFF 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"00FF FEEE EEEE EEEE EEEE EEEE EEEE FFF0 00FF F000 0000 0000"
	$"0000 0000 0000 FFF0 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0 00FF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFF0"
};

resource 'icl8' (gxPrintingDriverBaseID + 4, sysheap, purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FF00 0000 0000 0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B"
	$"2BFF FFFF 0000 0000 0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 FF2B 2B2B FFFF FF00 0000 0000 FFFF FFF5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B 2B2B 2BFF FFFF 0000"
	$"0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FFFF"
	$"FFFF FFFF FFFF FF00 0000 FFFF FFF5 F5F5 FFFF FFFF FFFF FFFF"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FF00 0000 FFFF FFF5 F5F5"
	$"FF48 4848 4848 48FF F5F5 F5F5 FFFF FFF5 F5F5 F5F5 FFFF FF00"
	$"0000 FFFF FFF5 F5F5 FF48 4848 4848 48FF F5F5 F5FF 1313 13FF"
	$"F5F5 F5F5 FFFF FF00 0000 FFFF FFF5 F5F5 FF48 4848 4848 48FF"
	$"F5F5 FF13 1313 1313 FFF5 F5F5 FFFF FF00 0000 FFFF FFF5 F5F5"
	$"FFFF FFFF FFFF FFFF F5F5 FF13 1313 1313 FFF5 F5F5 FFFF FF00"
	$"0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF13 1313 1313"
	$"FFF5 F5F5 FFFF FF00 0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5FF"
	$"F5F5 F5FF 1313 13FF F5F5 F5F5 FFFF FF00 00FF FFF9 FFF5 F5F5"
	$"F5F5 F5F5 F5F5 FF7F FFF5 F5F5 FFFF FFF5 F5F5 F5F5 FFFF FF00"
	$"FFFF FFF5 FFF5 F5F5 F5F5 F5F5 F5FF 7F7F 7FFF F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFFF FFFF FFFF FFF7 FFF5 F5F5 F5F5 F5F5 FF7F 7F7F"
	$"7F7F FFF5 F5F5 F5F5 F5F5 F5F5 FFFF FFFF FFFF FFF9 FFF5 F5F5"
	$"F5F5 F5FF 7F7F 7F7F 7F7F 7FFF F5F5 F5F5 F5F5 F5F5 FFFF FFFF"
	$"FFFF FFFC FFF5 F5F5 F5F5 F5F5 FF7F 7F7F 7F7F FFF5 F5F5 F5F5"
	$"F5F5 F5F5 FFFF FF00 00FF FFFF FFF5 F5F5 F5F5 F5F5 F5FF 7F7F"
	$"7FFF F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FF00 00FF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"00FF FFF9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 F9FF FF00 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF FFFF FFFF FFF7 F7F7 F7F7"
	$"F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 F7FF FFFF"
	$"FFFF FFF9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 F9FF FFFF 00FF FFFC FCFC FCFC FCFC FCFC FCFC FCFC"
	$"FCFC FCFC FCFC FCFC FCFC FCFC FCFF FFFF 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 FFFF FFFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB"
	$"FBFB FBFB FFFF FF00 0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FF00 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FF00 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};

// current, shared
resource 'ICN#' (gxPrintingDriverBaseID + 5, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"07FF FE00 07FF FF00 07FF FF80 0700 05C0 0700 04E0 070F 8470"
		$"0708 9C38 1F08 A7FC 3F0F A21E 3900 2217 3900 9C17 3901 4017"
		$"1F02 201E 1FFF FFFE 3800 0007 3800 0007 3800 0007 3FFF FFFF"
		$"1FFF FFFF 1F00 001E 1FFF FFFE 1FFF FF7C 0FFF FF78 0000 03E0"
		$"0000 0220 0000 0220 0000 03E0 0000 02A0 BFFF FC9D 0000 0140"
		$"BFFF FE3D",
		/* [2] */
		$"07FF FE00 07FF FF00 07FF FF80 07FF FFC0 07FF FFE0 07FF FFF0"
		$"07FF FFF8 1FFF FFFC 3FFF FFFE 3FFF FFFF 3FFF FFFF 3FFF FFFF"
		$"1FFF FFFE 1FFF FFFE 3FFF FFFF 3FFF FFFF 3FFF FFFF 3FFF FFFF"
		$"1FFF FFFF 1FFF FFFE 1FFF FFFE 1FFF FFFC 0FFF FFF8 0000 03E0"
		$"0000 03E0 0000 03E0 0000 03E0 0000 03E0 FFFF FFFF FFFF FF7F"
		$"FFFF FE3F"
	}
};

resource 'icl4' (gxPrintingDriverBaseID + 5, sysheap, purgeable) {
	$"0000 0FFF FFFF FFFF FFFF FFF0 0000 0000 0000 0FFF FFFF FFFF"
	$"FFFF FFFF 0000 0000 0000 0FFF FFFF FFFF FFFF FFFF F000 0000"
	$"0000 0FFF 0000 0000 0000 0FCF FF00 0000 0000 0FFF 0000 0000"
	$"0000 0FCC FFF0 0000 0000 0FFF 0000 FFFF F000 0FCC CFFF 0000"
	$"0000 0FFF 0000 F777 F00F FFCC CCFF F000 000F FFFF 0000 F878"
	$"F0F4 4FFF FFFF FF00 00FF FFFF 0000 FFFF F0F4 44F0 000F FFF0"
	$"00FF FDDF 0000 0000 00F4 44F0 000F DFFF 00FF F00F 0000 0000"
	$"F00F FF00 000F DFFF 00FF FDDF 0000 000F 5F00 0000 000F DFFF"
	$"000F FEEF 0000 00F5 55F0 0000 000F EFF0 000F FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFF0 00FF FDDD DDDD DDDD DDDD DDDD DDDD DFFF"
	$"00FF F000 0000 0000 0000 0000 0000 0FFF 00FF FCCC CCCC CCCC"
	$"CCCC CCCC CCCC CFFF 00FF FDDD DDDD DDDD DDDD DDDD DDDD DFFF"
	$"000F FFFF FFFF FFFF FFFF FFFF FFFF FFFF 000F FFFF EEEE EEEE"
	$"EEEE EEEE EEEF FFF0 000F FFFF FFFF FFFF FFFF FFFD CDFF FFF0"
	$"000F FFFF FFFF FFFF FFFF FFFE CEFF FF00 0000 FFFF FFFF FFFF"
	$"FFFF FFFE CEFF F000 0000 0000 0000 0000 0000 00FF FFF0 0000"
	$"0000 0000 0000 0000 0000 00FC CCF0 0000 0000 0000 0000 0000"
	$"0000 00FC CCF0 0000 0000 0000 0000 0000 0000 00FF FFF0 0000"
	$"0000 0000 0000 0000 0000 00FC FCE0 0000 DDDD DDDD DDDD DDDD"
	$"EEEE EECC FCCE EEED CCCC CCCC CCCC CCCC CCCC CCCF 0FCC CCCC"
	$"DDDD DDDD DDDE EEEE EEEE EFF0 00FF EEEE"
};

resource 'icl8' (gxPrintingDriverBaseID + 5, sysheap, purgeable) {
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 0000 0000 0000 0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF 0000 0000 0000 0000 0000 0000 00FF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00 0000 0000 0000"
	$"0000 0000 00FF FFFF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 2BFF"
	$"FFFF 0000 0000 0000 0000 0000 00FF FFFF F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B FFFF FF00 0000 0000 0000 0000 00FF FFFF"
	$"F5F5 F5F5 FFFF FFFF FFF5 F5F5 F5FF 2B2B 2BFF FFFF 0000 0000"
	$"0000 0000 00FF FFFF F5F5 F5F5 FF48 4848 FFF5 F5FF FFFF 2B2B"
	$"2B2B FFFF FF00 0000 0000 00FF FFFF FFFF F5F5 F5F5 FF48 4848"
	$"FFF5 FF13 13FF FFFF FFFF FFFF FFFF 0000 0000 FFFF FFFF FFFF"
	$"F5F5 F5F5 FFFF FFFF FFF5 FF13 1313 FFF5 F5F5 F5FF FFFF FF00"
	$"0000 FFFF FFF9 F9FF F5F5 F5F5 F5F5 F5F5 F5F5 FF13 1313 FFF5"
	$"F5F5 F5FF F9FF FFFF 0000 FFFF FFF5 F5FF F5F5 F5F5 F5F5 F5F5"
	$"FFF5 F5FF FFFF F5F5 F5F5 F5FF F9FF FFFF 0000 FFFF FFF9 F9FF"
	$"F5F5 F5F5 F5F5 F5FF 7FFF F5F5 F5F5 F5F5 F5F5 F5FF F9FF FFFF"
	$"0000 00FF FFFC FCFF F5F5 F5F5 F5F5 FF7F 7F7F FFF5 F5F5 F5F5"
	$"F5F5 F5FF FCFF FF00 0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00 0000 FFFF FFF9 F9F9"
	$"F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9FF FFFF"
	$"0000 FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF FFFF 0000 FFFF FFF8 F8F8 F8F8 F8F8 F8F8 F8F8"
	$"F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8FF FFFF 0000 FFFF FFFA FAFA"
	$"FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFF FFFF"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF 0000 00FF FFFF FFFF FBFB FBFB FBFB FBFB"
	$"FBFB FBFB FBFB FBFB FBFB FBFF FFFF FF00 0000 00FF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFA 2BFA FFFF FFFF FF00"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFC"
	$"2BFC FFFF FFFF 0000 0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFC 2BFD FFFF FF00 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 FFFF FFFF FF00 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 FF2B"
	$"2B2B FF00 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 FF2B 2B2B FF00 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 FFFF FFFF FF00 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 FF2B"
	$"FE2B FD00 0000 0000 F6F6 F6F6 F7F7 F7F8 F8F8 F9F9 F9FA FAFA"
	$"FBFB FBFC FCFC 2B2B FE2B 2BFD FDFC FCFA F5F5 F5F5 F5F5 F6F6"
	$"F6F6 F6F6 F62B 2B2B 2B2B 2B2B 2B2B 2BFE 00FE 2B2B 2B2B 2B2B"
	$"F7F7 F8F8 F8F9 F9F9 FAFA FAFB FBFB FCFC FCFD FDFD FEFE FE00"
	$"0000 FEFE FDFD FCFC"
};

// inactive
resource 'ICN#' (gxPrintingDriverBaseID + 6, sysheap, purgeable) 
{
	{	/* array: 2 elements */
		/* [1] */
		$"6FFF FE06 F800 030F F800 029F 7C00 027E 3E00 027C 1F00 02F8"
		$"0F80 03F8 0FFF 03E8 0BE1 07C8 09F1 0F88 08F9 1F88 087F 3E88"
		$"083E 7C88 381F F90E 580F FE0D 4807 E009 4807 E009 680F F00B"
		$"581F F80D 383E 7C0E 3FFF FFFE 54FD 5F55 41F0 0F81 43E0 07C1"
		$"6FEA ABEB 5FD5 55F5 3FFF FFFE 3FFF FFFC 7C00 003E F800 001F"
		$"FFFF FFFF 6000 0006",
		/* [2] */
		$"6FFF FE06 FFFF FF0F FFFF FF9F 7FFF FFFE 3FFF FFFC 1FFF FFF8"
		$"0FFF FFF8 0FFF FFF8 0FFF FFF8 0FFF FFF8 0FFF FFF8 0FFF FFF8"
		$"0FFF FFF8 3FFF FFFE 7FFF FFFF 7FFF FFFF 7FFF FFFF 7FFF FFFF"
		$"7FFF FFFF 3FFF FFFE 3FFF FFFE 7FFF FFFF 7FFF FFFF 7FFF FFFF"
		$"7FFF FFFF 7FFF FFFF 3FFF FFFE 3FFF FFFC 7FFF FFFE FFFF FFFF"
		$"FFFF FFFF 6000 0006"
	}
};

resource 'icl8' (gxPrintingDriverBaseID + 6, sysheap, purgeable) 
{
	$"00FF FF00 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 0000 00FF FF00 FFFF FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 FFFF 0000 0000 FFFF FFFF FFFF FFFF FFF5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B FF00 00FF FFFF FFFF"
	$"00FF FFFF FFFF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B"
	$"2BFF FFFF FFFF FF00 0000 FFFF FFFF FFF5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 FF2B 2BFF FFFF FFFF 0000 0000 00FF FFFF FFFF"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FF2B FFFF FFFF FF00 0000"
	$"0000 0000 FFFF FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FFFF"
	$"FFFF FFFF FF00 0000 0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"F5F5 F5F5 F5F5 FFFF FFFF FFF5 FF00 0000 0000 0000 FFF5 FFFF"
	$"FFFF FF48 4848 48FF F5F5 F5F5 F5FF FFFF FFFF F5F5 FF00 0000"
	$"0000 0000 FFF5 F5FF FFFF FFFF 4848 48FF F5F5 F5F5 FFFF FFFF"
	$"FFF5 F5F5 FF00 0000 0000 0000 FFF5 F5F5 FFFF FFFF FF48 48FF"
	$"F5F5 F5FF FFFF FFFF FFF5 F5F5 FF00 0000 0000 0000 FFF5 F5F5"
	$"00FF FFFF FFFF FFFF F5F5 FFFF FFFF FF13 FFF5 F5F5 FF00 0000"
	$"0000 0000 FFF5 F5F5 F5F5 FFFF FFFF FFF5 F5FF FFFF FFFF 1313"
	$"FFF5 F5F5 FF00 0000 0000 FFFF FFF5 F5F5 F5F5 F5FF FFFF FFFF"
	$"FFFF FFFF FF13 13FF F5F5 F5F5 FFFF FF00 00FF F9F9 FFF5 F5F5"
	$"F5F5 F5F5 FFFF FFFF FFFF FFFF FFFF FFF5 F5F5 F5F5 FFF9 F9FF"
	$"00FF F5F5 FFF5 F5F5 F5F5 F5F5 F5FF FFFF FFFF FFF5 F5F5 F5F5"
	$"F5F5 F5F5 FFF9 F5FF 00FF F7F7 FFF5 F5F5 F5F5 F5F5 F5FF FFFF"
	$"FFFF FFF5 F5F5 F5F5 F5F5 F5F5 FFF9 F7FF 00FF F9F9 FFF5 F5F5"
	$"F5F5 F5F5 FFFF FFFF FFFF FFFF F5F5 F5F5 F5F5 F5F5 FFF9 F9FF"
	$"00FF FCFC FFF5 F5F5 F5F5 F5FF FFFF FFFF FFFF FFFF FFF5 F5F5"
	$"F5F5 F5F5 FFFC FCFF 0000 FFFF FFF5 F5F5 F5F5 FFFF FFFF FF7F"
	$"7FFF FFFF FFFF F5F5 F5F5 F5F5 FFFF FF00 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"00FF F9F9 F9F9 F9F9 FFFF FFFF FFF9 F9F9 F9F9 F9FF FFFF FFFF"
	$"F9F9 F9F9 F9F9 F9FF 00FF F5F5 F5F5 F5FF FFFF FFFF F5F5 F5F5"
	$"F5F5 F5F5 FFFF FFFF FFF5 F5F5 F5F5 F5FF 00FF F7F7 F7F7 FFFF"
	$"FFFF FFF7 F7F7 F7F7 F7F7 F7F7 F7FF FFFF FFFF F7F7 F7F7 F7FF"
	$"00FF F9F9 F9FF FFFF FFFF F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 FFFF"
	$"FFFF FFF9 F9F9 F9FF 00FF FCFC FFFF FFFF FFFC FCFC FCFC FCFC"
	$"FCFC FCFC FCFC FCFF FFFF FFFF FCFC FCFF 0000 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"0000 FFFF FFFF FFFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB"
	$"FBFF FFFF FFFF 0000 00FF FFFF FFFF F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FFFF FF00 FFFF FFFF FFF5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF 00FF FF00 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 00FF FF"
};

resource 'icl4' (gxPrintingDriverBaseID + 6, sysheap, purgeable) 
{
	$"0FF0 FFFF FFFF FFFF FFFF FFF0 0000 0FF0 FFFF F0C0 C0C0 C0C0"
	$"C0C0 C0FF 0000 FFFF FFFF FC0C 0C0C 0C0C 0C0C 0CFC F00F FFFF"
	$"0FFF FFC0 C0C0 C0C0 C0C0 C0FC CFFF FFF0 00FF FFFC 0C0C 0C0C"
	$"0C0C 0CFC CFFF FF00 000F FFFF C0C0 C0C0 C0C0 C0FC FFFF F000"
	$"0000 FFFF FC0C 0C0C 0C0C 0CFF FFFF F000 0000 FFFF FFFF FFFF"
	$"C0C0 C0FF FFF0 F000 0000 FCFF FFF8 777F 0C0C 0FFF FF0C F000"
	$"0000 F0CF FFFF 778F C0C0 FFFF F0C0 F000 0000 FC0C FFFF F87F"
	$"0C0F FFFF FC0C F000 0000 F0C0 CFFF FFFF C0FF FFF4 F0C0 F000"
	$"0000 FC0C 0CFF FFFC 0FFF FF44 FC0C F000 00FF F0C0 C0CF FFFF"
	$"FFFF F44F C0C0 FFF0 0FDD FC0C 0C0C FFFF FFFF FFFC 0C0C FEDF"
	$"0F00 F0C0 C0C0 CFFF FFF0 C0C0 C0C0 FE0F 0FCC FC0C 0C0C 0FFF"
	$"FFFC 0C0C 0C0C FECF 0FDD F0C0 C0C0 FFFF FFFF C0C0 C0C0 FEDF"
	$"0FEE FC0C 0C0F FFFF FFFF FC0C 0C0C FEEF 00FF F0C0 C0FF FFF5"
	$"5FFF FFC0 C0C0 FFF0 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"0FDD DDDD FFFF FDDD DDDF FFFF DDDD DDDF 0F00 000F FFFF 0000"
	$"0000 FFFF F000 000F 0FCC CCFF FFFC CCCC CCCC CFFF FFCC CCCF"
	$"0FDD DFFF FFDD DDDD DDDD DDFF FFFD DDDF 0FEE FFFF FEEE EEEE"
	$"EEEE EEEF FFFF EEEF 00FF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
	$"00FF FFFE EEEE EEEE EEEE EEEE EFFF FF00 0FFF FFC0 C0C0 C0C0"
	$"C0C0 C0C0 C0FF FFF0 FFFF FC0C 0C0C 0C0C 0C0C 0C0C 0C0F FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0FF0 0000 0000 0000"
	$"0000 0000 0000 0FF0"
};

resource 'ics#' (gxPrintingDriverBaseID + 6, sysheap, purgeable) 
{
	{	/* array: 2 elements */
		/* [1] */
		$"7FE2 E037 702E 383C 3F3C 2F7C 2FEC 23F4 E3C7 A7E5 EF77 FFFF"
		$"B81D FFFF E007 7FFE",
		/* [2] */
		$"7FE2 FFF7 7FFE 3FFC 3FFC 3FFC 3FFC 3FFC FFFF FFFF FFFF FFFF"
		$"FFFF FFFF FFFF 7FFE"
	}
};

resource 'ics8' (gxPrintingDriverBaseID + 6, sysheap, purgeable) 
{
	$"00FF FFFF FFFF FFFF FFFF FF00 0000 FF00 FFFF FFF5 F5F5 F5F5"
	$"F5F5 FFFF 00FF FFFF 00FF FFFF F5F5 F5F5 F5F5 FF2B FFFF FF00"
	$"0000 FFFF FFF5 F5F5 F5F5 FFFF FFFF 0000 0000 FFFF FFFF FFFF"
	$"F5F5 FFFF FFFF 0000 0000 FFF5 FFFF FFFF F5FF FFFF FFFF 0000"
	$"0000 FFF5 FFFF FFFF FFFF FF13 FFFF 0000 0000 FFF5 F5F5 FFFF"
	$"FFFF FFFF F5FF 0000 FFFF FFF5 F5F5 FFFF FFFF F5F5 F5FF FFFF"
	$"FFF8 FFF5 F5FF FFFF FFFF FFF5 F5FF F8FF FFFF FFF5 FFFF FFFF"
	$"F5FF FFFF F5FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFF8 FFFF FFF8 F8F8 F8F8 F8FF FFFF F8FF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFF5 F5F5 F5F5 F5F5 F5F5 F5FF FFFF"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};

resource 'ics4' (gxPrintingDriverBaseID + 6, sysheap, purgeable) 
{
	$"0FFF FFFF FFF0 00F0 FFFC 0C0C 0CFF 0FFF 0FFF C0C0 C0FC FFF0"
	$"00FF FC0C 0CFF FF00 00FF FFFF C0FF FF00 00FC FFFF 0FFF FF00"
	$"00F0 FFFF FFF4 FF00 00FC 0CFF FFFF 0F00 FFF0 C0FF FFC0 CFFF"
	$"FCFC 0FFF FFFC 0FCF FFF0 FFFF CFFF CFFF FFFF FFFF FFFF FFFF"
	$"FCFF FCCC CCCF FFCF FFFF FFFF FFFF FFFF FFF0 C0C0 C0C0 CFFF"
	$"0FFF FFFF FFFF FFF0"
};

// puzzle piece
resource 'ICN#' (gxPrintingDriverBaseID + 7, sysheap, purgeable) 
{
	{	/* array: 2 elements */
		/* [1] */
		$"00FF FF00 0100 0080 0080 0100 7F80 01FE 8000 0001 83FF F001"
		$"8200 1801 8200 1401 8200 1201 823E 1101 8222 7081 8222 9FC1"
		$"8E3E 8871 9200 8849 9202 7049 9205 0049 9E08 8079 8FFF FFF1"
		$"9000 0009 9000 0009 9000 0009 9FFF FFF9 8FFF FFF1 83FF FFC1"
		$"8200 0041 83FF FFC1 8000 0001 8000 0001 80FF FF01 8080 0101"
		$"8040 0201 7FE0 07FE",
		/* [2] */
		$"00FF FF00 01FF FF80 00FF FF00 7FFF FFFE FFFF FFFF FFFF FFFF"
		$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
		$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
		$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
		$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF80 01FF"
		$"FFC0 03FF 7FE0 07FE"
	}
};

resource 'icl8' (gxPrintingDriverBaseID + 7, sysheap, purgeable) 
{
	$"0000 0000 0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"0000 0000 0000 0000 0000 0000 0000 00FF 0000 0000 0000 0000"
	$"0000 0000 0000 0000 FF00 0000 0000 0000 0000 0000 0000 0000"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF 0000 0000 0000 0000"
	$"00FF FFFF FFFF FFFF FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"FFFF FFFF FFFF FF00 FF00 0000 0000 0000 0000 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 0000 0000 0000 00FF FF00 F6F6 F6F6 FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"FF00 F6F6 F6F6 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5FF FFF6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF FF00 F6F6 F6F6 FFF5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5FF 2BFF F6F6 F6F6 F6F6 F6F6 F6FF FF00 F6F6 F6F6 FFF5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 2B2B FFF6 F6F6 F6F6 F6F6 F6FF"
	$"FF00 F6F6 F6F6 FFF5 F5F5 FFFF FFFF FFF5 F5F5 F5FF 2B2B 2BFF"
	$"F6F6 F6F6 F6F6 F6FF FF00 F6F6 F6F6 FFF5 F5F5 FF48 4848 FFF5"
	$"F5FF FFFF 2B2B 2B2B FFF6 F6F6 F6F6 F6FF FF00 F6F6 F6F6 FFF5"
	$"F5F5 FF48 4848 FFF5 FF13 13FF FFFF FFFF FFFF F6F6 F6F6 F6FF"
	$"FF00 F6F6 FFFF FFF5 F5F5 FFFF FFFF FFF5 FF13 1313 FFF5 F5F5"
	$"F5FF FFFF F6F6 F6FF FF00 F6FF F9F9 FFF5 F5F5 F5F5 F5F5 F5F5"
	$"FF13 1313 FFF5 F5F5 F5FF F9F9 FFF6 F6FF FF00 F6FF F5F5 FFF5"
	$"F5F5 F5F5 F5F5 FFF5 F5FF FFFF F5F5 F5F5 F5FF F9F5 FFF6 F6FF"
	$"FF00 F6FF F9F9 FFF5 F5F5 F5F5 F5FF 7FFF F5F5 F5F5 F5F5 F5F5"
	$"F5FF F9F9 FFF6 F6FF FF00 F6FF FCFC FFF5 F5F5 F5F5 FF7F 7F7F"
	$"FFF5 F5F5 F5F5 F5F5 F5FF FCFC FFF6 F6FF FF00 F6F6 FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF F6F6 F6FF"
	$"FF00 F6FF F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 FFF6 F6FF FF00 F6FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 FFF6 F6FF FF00 F6FF F8F8 F8F8"
	$"F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 F8F8 FFF6 F6FF"
	$"FF00 F6FF FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA FAFA"
	$"FAFA FAFA FFF6 F6FF FF00 F6F6 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF F6F6 F6FF FF00 F6F6 F6F6 FFFB"
	$"FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFB FBFF F6F6 F6F6 F6FF"
	$"FF00 F6F6 F6F6 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5FF F6F6 F6F6 F6FF FF00 F6F6 F6F6 FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF F6F6 F6F6 F6FF FF00 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF FF00 F6F6 F6F6 F6F6 FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF F6F6 F6F6 F6F6 F6FF FF00 F6F6 F6F6 F6F6"
	$"FF00 0000 0000 0000 0000 0000 0000 00FF 00F6 F6F6 F6F6 F6FF"
	$"FF00 F6F6 F6F6 F6F6 F6FF 0000 0000 0000 0000 0000 0000 FF00"
	$"F6F6 F6F6 F6F6 F6FF 00FF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FF"
};

resource 'icl4' (gxPrintingDriverBaseID + 7, sysheap, purgeable) 
{
	$"0000 0000 FFFF FFFF FFFF FFFF 0000 0000 0000 000F 0000 0000"
	$"0000 0000 F000 0000 0000 0000 F0C0 C0C0 C0C0 C0CF 0000 0000"
	$"0FFF FFFF F00C 0C0C 0C0C 0C0F FFFF FFF0 F000 0000 00C0 C0C0"
	$"C0C0 C0C0 0000 000F F00C 0CFF FFFF FFFF FFFF 0C0C 0C0C 0C0F"
	$"F0C0 C0F0 0000 0000 000F F0C0 C0C0 C0CF F00C 0CF0 0000 0000"
	$"000F CF0C 0C0C 0C0F F0C0 C0F0 0000 0000 000F CCF0 C0C0 C0CF"
	$"F00C 0CF0 00FF FFF0 000F CCCF 0C0C 0C0F F0C0 C0F0 00F7 77F0"
	$"0FFF CCCC F0C0 C0CF F00C 0CF0 00F8 78F0 F44F FFFF FF0C 0C0F"
	$"F0C0 FFF0 00FF FFF0 F444 F000 0FFF C0CF F00F DDF0 0000 0000"
	$"F444 F000 0FDD FC0F F0CF 00F0 0000 00F0 0FFF 0000 0FD0 F0CF"
	$"F00F DDF0 0000 0F5F 0000 0000 0FDD FC0F F0CF EEF0 0000 F555"
	$"F000 0000 0FEE F0CF F00C FFFF FFFF FFFF FFFF FFFF FFFF 0C0F"
	$"F0CF DDDD DDDD DDDD DDDD DDDD DDDD F0CF F00F 0000 0000 0000"
	$"0000 0000 0000 FC0F F0CF CCCC CCCC CCCC CCCC CCCC CCCC F0CF"
	$"F00F DDDD DDDD DDDD DDDD DDDD DDDD FC0F F0C0 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF C0CF F00C 0CFE EEEE EEEE EEEE EEEE EF0C 0C0F"
	$"F0C0 C0F0 0000 0000 0000 0000 0FC0 C0CF F00C 0CFF FFFF FFFF"
	$"FFFF FFFF FF0C 0C0F F0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0CF"
	$"F00C 0C0C 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F F0C0 C0C0 FFFF FFFF"
	$"FFFF FFFF C0C0 C0CF F00C 0C0C F000 0000 0000 000F 0C0C 0C0F"
	$"F0C0 C0C0 CF00 0000 0000 00F0 C0C0 C0CF 0FFF FFFF FFF0 0000"
	$"0000 0FFF FFFF FFF0"
};

resource 'ics#' (gxPrintingDriverBaseID + 7, sysheap, purgeable) 
{
	{	/* array: 2 elements */
		/* [1] */
		$"1FF8 700E 9FC1 9061 9751 9579 9709 9009 B08D B14D 9FF9 A005"
		$"BFFD 8001 8FF1 7C3E",
		/* [2] */
		$"1FF8 7FFE FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
		$"FFFF FFFF FFFF 7C3E"
	}
};

resource 'ics8' (gxPrintingDriverBaseID + 7, sysheap, purgeable) 
{
	$"0000 00FF FFFF FFFF FFFF FFFF FF00 0000 00FF FFFF F6F6 F6F6"
	$"F6F6 F6F6 FFFF FF00 FFF6 F6FC FCFC FCFC FCFC F6F6 F6F6 F6FF"
	$"FFF6 F6FC F5F5 F5F5 F5FC FCF6 F6F6 F6FF FFF6 F6FC F5FF FFFF"
	$"F5FC F5FC F6F6 F6FF FFF6 F6FC 00FF 48FF 00FC FCFC FCF6 F6FF"
	$"FFF6 F6FC F5FF FFFF F5F5 F5F5 FCF6 F6FF FFF6 F6FC 0000 F500"
	$"0000 F5F5 FCF6 F6FF FFF6 FFFF F5F5 F5F5 FFF5 F5F5 FCFF F6FF"
	$"FFF6 FFFF 0000 F5FF 7FFF F5F5 FCFF F6FF FFF6 F9FF FFFF FFFF"
	$"FFFF FFFF FFF9 F6FF FFF6 FFF7 F7F7 F7F7 F7F7 F7F7 F7FF F6FF"
	$"FFF6 FFFF FFFF FFFF FFFF FFFF FFFF F6FF FFF6 F6F6 F6F6 F6F6"
	$"F6F6 F6F6 F6F6 F6FF FFF6 F6F6 FFFF FFFF FFFF FFFF F6F6 F6FF"
	$"00FF FFFF FFFF 0000 0000 FFFF FFFF FF"
};

resource 'ics4' (gxPrintingDriverBaseID + 7, sysheap, purgeable) 
{
	$"000F FFFF FFFF F000 0FFF 0C0C 0C0C FFF0 F0CE EEEE EEC0 C0CF"
	$"FC0E 0000 0EEC 0C0F F0CE 0FFF 0E0E C0CF FC0E 0F7F 0EEE EC0F"
	$"F0CE 0FFF 0000 E0CF FC0E 0000 0000 EC0F F0FF 0000 F000 EFCF"
	$"FCFF 000F 5F00 EF0F F0DF FFFF FFFF FDCF FCFC CCCC CCCC CF0F"
	$"F0FF FFFF FFFF FFCF FC0C 0C0C 0C0C 0C0F F0C0 FFFF FFFF C0CF"
	$"0FFF FF00 00FF FFF0"
};

/* ------------------------------------------------------------------------ */
/* RESOURCES FOR PAPERTYPES */
/* ------------------------------------------------------------------------ */

resource 'ptyp' (gxPrintingDriverBaseID +10, sysheap, purgeable)	// US Letter
{
	"US Letter",
	0x0,
	0x0,
	0x2400000,
	0x2FB3333,
	-0x120000,
	-0xE6666,
	0x2520000,
	0x3099999,
	usLetterBase,
	DriverCreator,
	inch,
	oldAndNewStylePaperType,
	isDefaultPaperType,
	{}
};

resource 'ptyp' (gxPrintingDriverBaseID +11, sysheap, purgeable)	// International Fanfold
{
	"International Fanfold",
	0x0,
	0x0,
	0x2520000,
	0x3600000,
	-0x120000,
	-0xE6666,
	0x2640000,
	0x36E6666,
	unknownBase,
	DriverCreator,
	mm,
	oldAndNewStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (gxPrintingDriverBaseID +12, sysheap, purgeable)	// US Legal
{
	"US Legal",
	0x0,
	0x0,
	0x2400000,
	0x3D33334,
	-0x120000,
	-0xE6666,
	0x2520000,
	0x3E1999A,
	usLegalBase,
	DriverCreator,
	inch,
	oldAndNewStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (gxPrintingDriverBaseID +13, sysheap, purgeable)	// Computer Paper
{
	"Computer Paper",
	0x0,
	0x0,
	0x3F00000,
	0x3180000,
	-0x120000,
	-0xE6666,
	0x4020000,
	0x3266666,
	unknownBase,
	DriverCreator,
	inch,
	oldAndNewStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (gxPrintingDriverBaseID +14, sysheap, purgeable)	// Envelope 
{
	"Envelope",
	0x0,
	0x0,
	0x2640000,
	0x1290000,
	-0x120000,
	-0xE6666,
	0x2760000,
	0x1376666,
	unknownBase,
	DriverCreator,
	inch,
	oldAndNewStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (gxPrintingDriverBaseID +15, sysheap, purgeable)	// A4 Letter 
{
	"A4 Letter",
	0x0,
	0x0,
	0x2400000,
	0x3180000,
	-0x120000,
	-0xE6666,
	0x2520000,
	0x3266666,
	a4LetterBase,
	DriverCreator,
	mm,
	oldAndNewStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (gxPrintingDriverBaseID +16, sysheap, purgeable)	// B5 Letter 
{
	"B5 Letter",
	0x0,
	0x0,
	0x1E000A8,
	0x2A600A8,
	-0x120000,
	-0xE6666,
	0x1F200A8,
	0x2B4670E,
	b5LetterBase,
	DriverCreator,
	mm,
	oldAndNewStylePaperType,
	notDefaultPaperType,
	{}
};

