/* ------------------------------------------------------------------------------
FILENAME
	OldAPIResources.r

DESCRIPTION
	This module contains the ImageWriter LQ resource constants and types that
	are needed to support the old (i.e. QuickDraw) application printing interface.
	
	Note: resources specific to the old API (e.g. print and style dialog resources)
	must retain the original resource ID's (e.g. -8192).  Don't set these resource
	ID's to be based upon printingDriverBaseID.  Also note that all resources
	should be designated to load into the system heap.
		
COPYRIGHT
	 Copyright Apple Computer, Inc. 1992-1996
	 All rights reserved. 

	12/20/93		dmh		Sync'd with the shipping 1.0b3 GX driver.
	 8/28/94		dmh		Sync'd with the shipping 1.0.1 GX driver.
	 6/14/96		cn			Updated to support Universal Interfaces 2.1.

-------------------------------------------------------------------------------- */

// System 7.0 Compatible
#define SystemSevenOrLater		1

// Include System Resource Definitions
#include "Types.r"
#include "SysTypes.r"
#include "BalloonTypes.r"

// Include the ImageWriter LQ driver constants that are used in the resources 
#include "LQResources.h"

// Include the public QuickDraw GX printing files 
#include "GXPrintingResTypes.r"


/*********************************************************************************
 *        							TYPES 				 						 					*
 *********************************************************************************/

type 'PREC' (0) 			// ImageWriter LQ - Old Style Print Record Definition that should be compatible with System 7.0 version of IW LQ driver
{
	integer = 1;							// iPrVersion
	
												// PrInfo
	integer;										//	iDev
	integer;										//	iVRes
	integer;										//	iHRes
	rect;											//	rPage

	rect;										//	rPaper

												//	PrStl
													// ******** START DUMMY ( READ ONLY ) STORAGE *********
	byte;											//	wDev refNum	
	boolean NoZoom, Zoom4X;					//	wDev 33% Reduction
	boolean NoDftBts, DftBts;				//	wDev fDraftBits
	boolean NoResSet, ResSet;				//	wDev fResSet
	boolean NoScroll, Scroll;				//	wDev fScroll
	boolean NoZoom, Zoom2X;					//	wDev f2XZoom
	boolean UniDir, BiDir;					//	wDev fBiDir		
	boolean Landscape, Portrait;			//	wDev fPortrait	
	boolean LowRes, HighRes;				//	wDev fHiRes		
													// ******** END DUMMY ( READ ONLY ) STORAGE ********* */
												
	integer;										//	iPageV		
	integer;										//	iPageH		
	byte;											//	bPort		
	byte Cut, Fanfold, MechCut, Other;	//	feed		

												//	PrInfoPT
	integer;										//	iDev		
	integer;										//	iVRes		
	integer;										//	iHRes		
	rect;											//	rPage		

												//	PrXInfo
	integer;										// iRowBytes	
	integer;										//	iBandV		
	integer;										//	iBandh		
	integer;										//	iDevBytes	
	integer;										//	iBands		
	byte dbl=-2, norm=0;						//	bPatScale	
	byte;											//	bULThick	
	byte;											//	bULOffset	
	byte;											//	bULShadow	
	byte scanTB, scanBT, scanLR, scanRL;//	scan		
	fill byte;									//	bXInfoX		

												//	PrJob		
	integer;										//	iFstPage	
	integer;										//	iLstPage	
	integer;										//	iCopies		
	byte draft, spool, User1, User2;		//	bJDocLoop	
	byte;											//	fFromUsr	
	longint;										//	pIdleProc	
	longint;										//	pFileName	
	integer;										//	iFileVol	
	byte;											//	bFileVers	
	byte;											//	bJobFlags	
	
												//	PrintX
	integer;										//	CSF Bin Flags 
	byte;											//	wDev refNum	
	boolean NoZoom, Zoom4X;					//	wDev 33% Reduction	
	boolean NoDftBts, DftBts;				//	wDev fDraftBits	
	boolean NoResSet, ResSet;				//	wDev fResSet	
	boolean NoScroll, Scroll;				//	wDev fScroll	
	boolean NoZoom, Zoom2X;					//	wDev f2XZoom	
	boolean UniDir, BiDir;					//	wDev fBiDir		
	boolean Landscape, Portrait;			//	wDev fPortrait	
	boolean LowRes, HighRes;				//	wDev fHiRes	

	fill word [17];							//	Make 120 byte total	
};


/*********************************************************************************
 *        							RESOURCES 			 						 					*
 *********************************************************************************/

/*************************** Old API Message Override Resource **************************/
// This overrideType resource tells the Printing Manager which old API messages we want to override, 
// as well as some new backward compatibility messages. 

resource gxOverrideType (gxDriverCompatibilityOverrideID, sysheap, purgeable)
{
	{
		gxPrValidate, 					kOldAPISegmentID, kFirstMsgJumpTableOffset,
		gxConvertPrintRecordTo, 	kOldAPISegmentID, kFirstMsgJumpTableOffset + 4,
		gxConvertPrintRecordFrom, kOldAPISegmentID, kFirstMsgJumpTableOffset + 8,
		gxPrintRecordToJob, 			kOldAPISegmentID, kFirstMsgJumpTableOffset + 12,
	};
};


/*************************** Old Application Compatibility Resource **************************/
// This custType resource is used by the Printing Manager.  It defines some basic options for 
// use with old applications.  Note: every driver should contain one of these resources. 

resource gxCustType (gxCustID, sysheap, purgeable)
{
	216,								// horizontal device resolution (dpi) 
	216, 								// vertical device resolution (dpi) 
	defaultUpDriver,				// type of "upDriver" to use 
	{3, 3},							// pattern stretching factor. Set to 3 (nearest thing to 216 dpi/72 dpi) 
	gxOptimizedTranslation + gxRasterTargetTranslation
};

/*************************** Old Application Resolution Resource **************************/

resource 'resl' (-8192, sysheap, purgeable)
{
	rangeType,
	gxDiscreteResolution, gxDiscreteResolution,
	gxDiscreteResolution, gxDiscreteResolution,
	{
	72, 	72;
	216, 	216;
	}
};


/************************** PREC Resources **************************/
// This PREC definition represents the default print record definition 
// for the old LaserWriter SC driver. This resource is accessed by the 
// Universal Dialog Handler. 

resource 'PREC' (0, sysheap, purgeable)			// PREC 0 - Default print record definition for the old LaserWriter SC driver 
{
	// PrInfo
	0,
	72,
	72,
	{0, 0, 744, 576},
	
	// rPaper
	{-25, -18, 767, 594},
	
	//PrStl
	5,
	NoZoom,
	NoDftBts,
	NoResSet,
	NoScroll,
	NoZoom,
	UniDir,
	Portrait,
	HighRes,
	1320,						//	In 1/120 inch measurements
	1020,
	0,
	Fanfold,
	
	// PrInfoPT
	0,
	216,
	216,
	{0, 0, 2232, 1728},
	
	// PrXInfo
	80,
	32,
	640,
	3200,
	24,
	norm,
	1,
	1,
	1,
	scanTB,
	
	// PrJob
	1,
	9999,
	1,
	spool,
	1,
	0,
	0,
	0,
	0,
	0,
	
	// PrintX
	5,
	5,
	NoZoom,
	NoDftBts,
	NoResSet,
	NoScroll,
	NoZoom,
	UniDir,
	Portrait,
	HighRes
};


// version string for old dialogs
resource 'STR ' (-8190, sysheap, purgeable) { "1.0" };

/************************** PrStlDialog Resources **************************/
// These dialog related resources define the old ImageWriter LQ's Style dialog. 
// These resources are manipulated by the Universal Dialog Handler 

resource 'DLOG' (gxStlDialogResID, sysHeap)			// Style Dialog DLOG 
{
	{30, 20, 174, 492},
	dBoxProc,
	invisible,
	noGoAway,
	0x1,
	gxStlDialogResID,
	"Stl",
	noAutoCenter
};

resource 'DITL' (gxStlDialogResID, sysheap, purgeable)			// Style Dialog DITL 
{
	{	/* array DITLarray: 22 elements */
		/* [1] */	{8, 404, 27, 464}, Button {enabled, "OK"},
		/* [2] */	{35, 404, 54, 464}, Button {enabled, "Cancel"},
		
		/* [3] */	{4, 4, 20, 384}, StaticText {disabled, "!0"},
		/* [4] */	{19, 5, 23, 393}, UserItem {disabled},
		
		/* [5] */	{24, 4, 40, 60}, StaticText {disabled, "Paper:"},
		/* [6] */	{24, 68, 40, 208}, RadioButton {enabled, "US Letter"},
		/* [7] */	{24, 218, 40, 383}, RadioButton {enabled, "A4 Letter"},
		/* [8] */	{39, 68, 55, 208}, RadioButton {enabled, "US Legal"},
		/* [9] */	{39, 218, 55, 383}, RadioButton {enabled, "International Fanfold"},
		/* [10] */	{54, 68, 70, 208}, RadioButton {enabled, "Computer Paper"},
		/* [11] */	{54, 218, 70, 383}, RadioButton {enabled, "Envelope (#10)"},
		
		/* [12] */	{74, 4, 90, 88}, StaticText {disabled, "Orientation"},
		/* [13] */	{93, 4, 125, 36}, UserItem {enabled},
		/* [14] */	{93, 44, 125, 76}, UserItem {enabled},
		
		/* [15] */	{74, 106, 90, 211}, StaticText {disabled, "Special Effects:"},
		/* [16] */	{106, 308, 122, 431}, RadioButton {enabled, "33 % Reduction"},
		/* [17] */	{122, 308, 138, 430}, RadioButton {enabled, "66 % Reduction"},
		/* [18] */	{90, 308, 106, 431}, RadioButton {enabled, "Full Size"},
		/* [19] */	{106, 221, 122, 307}, StaticText {disabled, "Reductions:"},
		/* [20] */	{74, 218, 90, 399}, CheckBox {enabled, "No Gaps Between Pages"},

		/* [21] */	{3, 341, 18, 365}, UserItem { enabled },			// area for version number 
		/* [22] */	{0, 394, 27, 464}, UserItem { disabled }			// user item for OK button 
	}
};

resource 'dctl' (gxStlDialogResID, sysheap, purgeable)				// Universal Dialog Handler dctl for Style Dialog 
{
	22,
	{
		Button {2, cancel},
		Frill {3, printerName},
		Frill {4, line},
		PaperSizes {0, 0, { 6, 7, 8, 9, 10, 11 }},
		Orientation {13, 14, 0, 0},
		Toggle {20, bBiggerPages},
		Cluster {user0, {16, 17, 18 }},
		Frill {21, version},
		Frill {22, default},
	}
};



// This resource is needed for proper use of color in Style dialog
resource 'dctb' (gxStlDialogResID, sysheap, purgeable)
{
	{	/* array ColorSpec: 0 elements */
	}
};


/************************** PrJobDialog Resources **************************/
// These dialog related resources define the old LaserWriter SC's Print dialog. 
// These resources are manipulated by the Universal Dialog Handler 

resource 'DLOG' (gxJobDialogResID, sysheap, purgeable) 			// Job Dialog DLOG 
{
	{28, 26, 156, 500},
	dBoxProc,
	invisible,
	noGoAway,
	0x1,
	gxJobDialogResID,
	"Job",
	noAutoCenter
};

resource 'DITL' (gxJobDialogResID, sysheap, purgeable)				//	Job Dialog DITL
{
	{	/* array DITLarray: 22 elements */
		/* [1] */	{8, 404, 27, 464}, Button {enabled, "Print"},
		/* [2] */	{35, 404, 54, 464}, Button {enabled, "Cancel"},
		
		/* [3] */	{4, 4, 20, 384}, StaticText {disabled, "!0"},
		/* [4] */	{19, 5, 23, 393}, UserItem {disabled},
		
		/* [5] */	{29, 4, 45, 88}, StaticText {disabled, "Quality:"},
		/* [6] */	{29, 100, 45, 188}, RadioButton {enabled, "Best"},
		/* [7] */	{29, 210, 45, 298}, RadioButton {enabled, "Faster"},
		/* [8] */	{29, 310, 45, 398}, RadioButton {enabled, "Draft"},
		
		/* [9] */	{79, 4, 95, 88}, StaticText {disabled, "Page Range:"},
		/* [10] */	{79, 100, 95, 188}, RadioButton {enabled, "All"},
		/* [11] */	{79, 210, 95, 270}, RadioButton {enabled, "From:"},
		/* [12] */	{79, 274, 95, 306}, EditText {disabled, ""},
		/* [13] */	{79, 314, 95, 342}, StaticText {enabled, "To:"},
		/* [14] */	{79, 342, 95, 374}, EditText {disabled, ""},
		
		/* [15] */	{104, 4, 119, 88}, StaticText {disabled, "Copies:"},
		/* [16] */	{104, 100, 119, 132}, EditText {disabled, "1"},
		
		/* [17] */	{54, 4, 70, 88}, StaticText {disabled, "Head Scan:"},
		/* [18] */	{54, 100, 70, 210}, RadioButton {enabled, "Bidirectional"},
		/* [19] */	{54, 210, 70, 332}, RadioButton {enabled, "Unidirectional"},
		
		/* [20] */	{64, 404, 84, 464}, Button {enabled, "Options"},

		/* [21] */	{3, 341, 18, 365}, UserItem { enabled },			// area for version number 
		/* [22] */	{0, 394, 27, 464}, UserItem { disabled }			// user item for OK button 
	}
};

resource 'dctl' (gxJobDialogResID, sysheap, purgeable)				// Universal Dialog Handler dctl for Job Dialog 
{
	22,
	{
		Button {2,	cancel},
		Frill {3, printerName},
		Frill {4, line},
		Cluster { quality, { 6, 7, 8 } },
		PageRange {10, 11, 12, 14 },
		Copies {16},
		Cluster { headMotion, { 19, 18 } },
		DialogBtn {20, kPrintOptionsDlgRsrcID, kPrintOptionsDlgRsrcID},
		Frill {21, version},
		Frill {22, default},
	}
};


// This resource is needed for proper use of color in Job dialog
resource 'dctb' (gxJobDialogResID, sysheap, purgeable)
{
	{	/* array ColorSpec: 0 elements */
	}
};

resource 'DLOG' (kPrintOptionsDlgRsrcID, sysheap, purgeable) 			// Print Options Dialog DLOG 
{
	{28, 26, 270, 500},
	dBoxProc,
	invisible,
	noGoAway,
	0x1,
	kPrintOptionsDlgRsrcID,
	"Paper",
	noAutoCenter
};

resource 'DITL' (kPrintOptionsDlgRsrcID, sysheap, purgeable)				//	Print Options Dialog DITL
{
	{	/* array DITLarray: 24 elements */
		/* [1] */	{4, 401, 24, 461}, Button {enabled, "OK"},
		/* [2] */	{34, 401, 54, 461}, Button {enabled, "Cancel"},
		
		/* [3] */	{4, 8, 20, 384}, StaticText {disabled, "!0, Paper Path Options"},
		/* [4] */	{19, 5, 23, 384}, UserItem {disabled},
		
		/* [5] */	{32, 96, 48, 192}, RadioButton {enabled, "Automatic"},
		/* [6] */	{32, 216, 48, 312}, RadioButton {enabled, "Hand Feed"},
		/* [7] */	{32, 8, 48, 88}, StaticText {disabled, "Paper Feed:"},
		
		/* [8] */	{72, 8, 88, 160}, StaticText {disabled, "Sheet Feeder Options:"},
		/* [9] */	{72, 200, 88, 240}, StaticText {disabled, "Bin 1"},
		/* [10] */	{72, 264, 88, 304}, StaticText {disabled, "Bin 2"},
		/* [11] */	{72, 328, 88, 368}, StaticText {disabled, "Bin 3"},
		/* [12] */	{96, 24, 112, 176}, StaticText {disabled, "First Sheet From:"},
		/* [13] */	{115, 23, 131, 183}, StaticText {disabled, "Remaining Sheets From:"},
		/* [14] */	{96, 208, 112, 240}, RadioButton {enabled, ""},
		/* [15] */	{96, 272, 112, 304}, RadioButton {enabled, ""},
		/* [16] */	{96, 336, 112, 368}, RadioButton {enabled, ""},
		/* [17] */	{115, 208, 131, 240}, RadioButton {enabled, ""},
		/* [18] */	{115, 272, 131, 304}, RadioButton {enabled, ""},
		/* [19] */	{115, 336, 131, 368}, RadioButton {enabled, ""},
		
		/* [20] */	{163, 9, 226, 455}, StaticText {disabled, "*PLEASE NOTE: Some applications treat each page of a print job as the first page."
																	"  These applications cannot take full advantage of the Sheet Feeder Options.  "
																	"Instead, they print each page from the bin you select in the First Sheet From option."},
		/* [21] */	{139, 7, 154, 457}, StaticText {disabled, "____________________________________________________________________"},
		/* [22] */	{142, 7, 157, 457}, StaticText {disabled, "____________________________________________________________________"},

		/* [23] */	{3, 341, 18, 365}, UserItem { enabled },			// area for version number 
		/* [24] */	{0, 395, 27, 465}, UserItem { disabled }			// user item for OK button 
	}
};

resource 'dctl' (kPrintOptionsDlgRsrcID, sysheap, purgeable)				// Universal Dialog Handler dctl for Print Options Dialog 
{
	24,
	{
		Button {2,	cancel},
		Frill {3, printerName},
		Frill {4, line},
		Cluster { feed, { 5, 6 } },
		Cluster { firstPage, { 14, 15, 16 } },
		Cluster { restPage, { 17, 18, 19 } },
		Frill {23, version},
		Frill {24, default},
	}
};


// This resource is needed for proper use of color in Print Options dialog
resource 'dctb' (kPrintOptionsDlgRsrcID, sysheap, purgeable)
{
	{	/* array ColorSpec: 0 elements */
	}
};


/*************************** Old API Paper Type Definitions **************************/
// These paper type definitions represent the settings of the old LaserWriter SC's paper 
// sizes. 

resource 'ptyp' (kOldPaperTypesID + 1, sysheap, purgeable)			// US Letter 
{
	"US Letter",
	
	// page rectangle 
	0x00000000,				// 8.0 x 10.33
	0x00000000,				
	0x02400000,				
	0x02E7C298,				

	// paper rectangle 
	0xFFEE0000,				// 8.5 x 11.0
	0xFFE7E14C,				
	0x02520000,				
	0x02FFE14C,				
	
	usLetterBase,
	kDrvrCreatorType,

	inch,
	oldStylePaperType,
	isDefaultPaperType,
	{}
};

resource 'ptyp' (kOldPaperTypesID + 2, sysheap, purgeable)			// A4 Letter 
{
	"A4 Letter",
	
	// page rectangle 
	0x00000000,				// 8.0 x 11.0 
	0x00000000,				
	0x02400000,				
	0x03180000,				

	// paper rectangle 
	0xFFF70000,				// 8.25 x 11.67
	0xFFE7E14C,				
	0x02490000,				
	0x03301EB4,				
	
	a4LetterBase,
	kDrvrCreatorType,

	inch,	
	oldStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (kOldPaperTypesID + 3, sysheap, purgeable)			// US Legal 
{
	"US Legal",
	
	// page rectangle 
	0x00000000,				// 8.0 x 13.33
	0x00000000,				
	0x02400000,					
	0x03BFC298,				

	// paper rectangle 
	0xFFEE0000,				// 8.5 x 14.0
	0xFFE7E14C,				
	0x02520000,				
	0x03D7E14C,				

	usLegalBase,
	kDrvrCreatorType,

	inch,	
	oldStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (kOldPaperTypesID + 4, sysheap, purgeable)			// International Fanfold 
{
	"International Fanfold",
	
	// page rectangle 
	0x00000000,					// 8.0 x 11.33 
	0x00000000,					
	0x02400000,					
	0x032FC298,					
	
	// paper rectangle 
	0xFFF70000,					// 8.25 x 12.0 
	0xFFE7E14C,					
	0x02490000,					
	0x0347E14C,					
	
	unknownBase,
	kDrvrCreatorType,

	inch,
	oldStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (kOldPaperTypesID + 5, sysheap, purgeable)			// Computer Paper 
{
	"Computer Paper",
	
	// page rectangle 
	0x00000000,				// 13.33 x 10.33 
	0x00000000,
	0x03BFC298,
	0x02E7C298,

	// paper rectangle 
	0xFFE7E14C,				//	14.0 x 11.0
	0xFFE7E14C,
	0x03D7E14C,
	0x02FFE14C,
	
	unknownBase,
	kDrvrCreatorType,

	inch,	
	oldStylePaperType,
	notDefaultPaperType,
	{}
};

resource 'ptyp' (kOldPaperTypesID + 6, sysheap, purgeable)			// Envelope (#10) 
{
	"Envelope (#10)",
	
	// page rectangle 
	0x00000000,					// 9.3333 x 3.3333 
	0x00000000,					
	0x029FFF58,					
	0x00EFFF58,					
	
	// paper rectangle 
	0xFFF9FFAC,					// 9.5 x 4.125 
	0xFFE37FAC,					
	0x02A5FFAC,					
	0x010C7FAC,					
	
	unknownBase,
	kDrvrCreatorType,

	inch,
	oldStylePaperType,
	notDefaultPaperType,
	{}
};


/*************************** Help Manager Resource Definitions **************************/
// These resources define the help text that will be displayed by the System 7 Help Manager
// if help is enabled and on eof the print dialogs is visible.


// The 'hwin' resource is used by the Help Manager and it links 'hdlg'
// resources to the dialogs to which they apply.  In this case, we have
// two 'hwin' list entries that reference the two old app. print dialogs.

resource 'hwin' (kHwinRsrcID, sysheap, purgeable)
{
	HelpMgrVersion,
	hmDefaultOptions,
	{
		gxStlDialogResID, 			'hdlg', 3, "Stl",		// Reference to PrStlDialog's 'hdlg' resource
		gxJobDialogResID, 			'hdlg', 3, "Job",		// Reference to PrJobDialog's 'hdlg' resource
		kPrintOptionsDlgRsrcID, 	'hdlg', 3, "Paper"	// Reference to Print Options dialog 'hdlg' resource
	}
};


// The 'hfdr' resource tells the Finder that we have help information it should
// display when the user moves the mouse over the ImageWriter LQ icon when help is enabled.

resource 'hfdr' (kHfdrRsrcID, sysheap, purgeable)
{
	HelpMgrVersion,
	hmDefaultOptions,
	0,					// Default balloon definition function
	0,					// Default variation code
	{
		HMSTRResItem
		{
			kFinderHelpSTRID	// Resource ID of help string ('STR ') to display
		}
	}
};


// This string resource contains the help text that will be displayed in the Finder if you
// move the mouse over the ImageWriter LQ icon when help is enabled.

resource 'STR ' (kFinderHelpSTRID, sysheap, purgeable)
{
	"ImageWriter LQ\n\n"
	"To enable your Macintosh to print documents on an ImageWriter LQ connected to a serial port, place this file in your Extensions folder.";
};


// This 'hdlg' resource tells the system what help strings to display when the user moves
// the mouse over the ImageWriter LQ's Style dialog with help enabled.

resource 'hdlg' (gxStlDialogResID, sysheap, purgeable) 
{
	HelpMgrVersion,
	0,										// Start help with the first item in the corresponding 'DITL'
	hmDefaultOptions,
	0,										// Default balloon definition function
	0,										// Ballon variation code - hang below and to the right
	HMStringResItem					//	This list references help strings to be used for any missing items
	{
		{0, 0},
		{0, 0, 0, 0},
		gxStlDialogResID, 1,
		gxStlDialogResID, 2,
		gxStlDialogResID, 3,
		gxStlDialogResID, 4
	},
	{	/* array HDialogArray: 22 elements */
		/* [1] */
		HMStringResItem				//	OK button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 5,
			gxStlDialogResID, 6,
			gxStlDialogResID, 7,
			gxStlDialogResID, 8
		},
		/* [2] */
		HMStringResItem				//	Cancel button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 9,
			gxStlDialogResID, 10,
			gxStlDialogResID, 11,
			gxStlDialogResID, 12
		},
		/* [3] */
		HMSkipItem						//	Driver name string
		{
		},
		/* [4] */
		HMSkipItem 						//	Horizontal line
		{
		},
		/* [5] */
		HMSkipItem 						//	Paper: string
		{
		},
		/* [6] */
		HMStringResItem 				//	US Letter button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 13,
			gxStlDialogResID, 14,
			gxStlDialogResID, 15,
			gxStlDialogResID, 16
		},
		/* [7] */
		HMStringResItem 				//	A4 Letter button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 17,
			gxStlDialogResID, 18,
			gxStlDialogResID, 19,
			gxStlDialogResID, 20
		},
		/* [8] */
		HMStringResItem 				// US Legal button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 21,
			gxStlDialogResID, 22,
			gxStlDialogResID, 23,
			gxStlDialogResID, 24
		},
		/* [9] */
		HMStringResItem 				//	International Fanfold button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 25,
			gxStlDialogResID, 26,
			gxStlDialogResID, 27,
			gxStlDialogResID, 28
		},
		/* [10] */
		HMStringResItem 				//	Computer Paper button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 29,
			gxStlDialogResID, 30,
			gxStlDialogResID, 31,
			gxStlDialogResID, 32
		},
		/* [11] */
		HMStringResItem 				//	Envelope (#10) button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 33,
			gxStlDialogResID, 34,
			gxStlDialogResID, 35,
			gxStlDialogResID, 36
		},
		/* [12] */						//	Orientation string
		HMSkipItem 
		{
		},
		/* [13] */
		HMStringResItem 				//	Portrait orientation icon
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 37,
			gxStlDialogResID, 38,
			gxStlDialogResID, 39,
			gxStlDialogResID, 40
		},
		/* [14] */
		HMStringResItem 				//	Landscape orientation icon
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 41,
			gxStlDialogResID, 42,
			gxStlDialogResID, 43,
			gxStlDialogResID, 44
		},
		/* [15] */
		HMSkipItem 						// Special Effects: string
		{
		},
		/* [16] */
		HMStringResItem 				//	33% Reduction button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 45,
			gxStlDialogResID, 46,
			gxStlDialogResID, 47,
			gxStlDialogResID, 48
		},
		/* [17] */
		HMStringResItem 				//	66% Reduction button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 49,
			gxStlDialogResID, 50,
			gxStlDialogResID, 51,
			gxStlDialogResID, 52
		},
		/* [18] */
		HMStringResItem 				//	Full size button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 53,
			gxStlDialogResID, 54,
			gxStlDialogResID, 55,
			gxStlDialogResID, 56
		},
		/* [19] */
		HMSkipItem 						// Reductions: string
		{
		},
		/* [20] */
		HMStringResItem 				//	No gaps between pages checkbox
		{
			{0, 0},
			{0, 0, 0, 0},
			gxStlDialogResID, 57,
			gxStlDialogResID, 58,
			gxStlDialogResID, 59,
			gxStlDialogResID, 60
		},
		/* [21] */
		HMSkipItem 						// Version number
		{
		},
		/* [22] */
		HMSkipItem 						// Default ring around the OK button
		{
		}
	}
};

// This 'hdlg' resource tells the system what help strings to display when the user moves
// the mouse over the ImageWriter LQ's Print dialog with help enabled.

resource 'hdlg' (gxJobDialogResID, sysheap, purgeable) 
{
	HelpMgrVersion,
	0,										// Start help with the first item in the corresponding 'DITL'
	hmDefaultOptions,
	0,										// Default balloon definition function
	0,										// Ballon variation code - hang below and to the right
	HMStringResItem					//	This list references help strings to be used for any missing items
	{
		{0, 0},
		{0, 0, 0, 0},
		gxJobDialogResID, 1,
		gxJobDialogResID, 2,
		gxJobDialogResID, 3,
		gxJobDialogResID, 4
	},
	{	/* array HDialogArray: 21 elements */
		/* [1] */
		HMStringResItem 				//	Print button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 5,
			gxJobDialogResID, 6,
			gxJobDialogResID, 7,
			gxJobDialogResID, 8
		},
		/* [2] */
		HMStringResItem 				//	Cancel button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 9,
			gxJobDialogResID, 10,
			gxJobDialogResID, 11,
			gxJobDialogResID, 12
		},
		/* [3] */
		HMSkipItem 						//	Print driver name string
		{
		},
		/* [4] */
		HMSkipItem 						//	Horizontal line
		{
		},
		/* [5] */
		HMSkipItem 						//	Quality: string
		{
		},
		/* [6] */
		HMStringResItem 				// Best quality button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 13,
			gxJobDialogResID, 14,
			gxJobDialogResID, 15,
			gxJobDialogResID, 16
		},
		/* [7] */
		HMStringResItem 				//	Faster quality button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 17,
			gxJobDialogResID, 18,
			gxJobDialogResID, 19,
			gxJobDialogResID, 20
		},
		/* [8] */
		HMStringResItem 				//	Draft quality button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 21,
			gxJobDialogResID, 22,
			gxJobDialogResID, 23,
			gxJobDialogResID, 24
		},
		/* [9] */
		HMSkipItem 						//	Page Range: string
		{
		},
		/* [10] */
		HMStringResItem 				//	All button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 25,
			gxJobDialogResID, 26,
			gxJobDialogResID, 27,
			gxJobDialogResID, 28
		},
		/* [11] */
		HMStringResItem 				//	From: button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 29,
			gxJobDialogResID, 30,
			gxJobDialogResID, 31,
			gxJobDialogResID, 32
		},
		/* [12] */
		HMStringResItem 				// Edittext of from field
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 33,
			gxJobDialogResID, 34,
			gxJobDialogResID, 35,
			gxJobDialogResID, 36
		},
		/* [13] */
		HMSkipItem						//	To: string
		{
		},
		/* [14] */
		HMStringResItem 				//	Edittext of to field
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 37,
			gxJobDialogResID, 38,
			gxJobDialogResID, 39,
			gxJobDialogResID, 40
		},
		/* [15] */
		HMSkipItem 						//	Copies: string
		{
		},
		/* [16] */
		HMStringResItem 				//	Number of copies EditText field
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 41,
			gxJobDialogResID, 42,
			gxJobDialogResID, 43,
			gxJobDialogResID, 44
		},
		/* [17] */
		HMSkipItem 						//	Head Scan: string
		{
		},
		/* [18] */
		HMStringResItem 				//	Bidirectional button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 45,
			gxJobDialogResID, 46,
			gxJobDialogResID, 47,
			gxJobDialogResID, 48
		},
		/* [19] */
		HMStringResItem 				//	Unidirectional button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 49,
			gxJobDialogResID, 50,
			gxJobDialogResID, 51,
			gxJobDialogResID, 52
		},
		/* [21] */
		HMStringResItem 				//	Options button
		{
			{0, 0},
			{0, 0, 0, 0},
			gxJobDialogResID, 53,
			gxJobDialogResID, 54,
			gxJobDialogResID, 55,
			gxJobDialogResID, 56
		},
		/* [20] */
		HMSkipItem 						//	Version number
		{
		},
		/* [20] */
		HMSkipItem 						//	Default ring around OK button
		{
		}
	}
};

// This 'hdlg' resource tells the system what help strings to display when the user moves
// the mouse over the ImageWriter LQ's Print Options dialog with help enabled.

resource 'hdlg' (kPrintOptionsDlgRsrcID, sysheap, purgeable) 
{
	HelpMgrVersion,
	0,										// Start help with the first item in the corresponding 'DITL'
	hmDefaultOptions,
	0,										// Default balloon definition function
	0,										// Ballon variation code - hang below and to the right
	HMStringResItem					//	This list references help strings to be used for any missing items
	{
		{0, 0},
		{0, 0, 0, 0},
		kPrintOptionsDlgRsrcID, 1,
		kPrintOptionsDlgRsrcID, 2,
		kPrintOptionsDlgRsrcID, 3,
		kPrintOptionsDlgRsrcID, 4
	},
	{	/* array HDialogArray: 23 elements */
		/* [1] */
		HMStringResItem 				//	OK button
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 5,
			kPrintOptionsDlgRsrcID, 6,
			kPrintOptionsDlgRsrcID, 7,
			kPrintOptionsDlgRsrcID, 8
		},
		/* [2] */
		HMStringResItem 				//	Cancel button
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 9,
			kPrintOptionsDlgRsrcID, 10,
			kPrintOptionsDlgRsrcID, 11,
			kPrintOptionsDlgRsrcID, 12
		},
		/* [3] */
		HMSkipItem 						//	Print driver string
		{
		},
		/* [4] */
		HMSkipItem 						//	Horizontal line
		{
		},
		/* [5] */
		HMStringResItem 				//	Automatic button
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 13,
			kPrintOptionsDlgRsrcID, 14,
			kPrintOptionsDlgRsrcID, 15,
			kPrintOptionsDlgRsrcID, 16
		},
		/* [6] */
		HMStringResItem 				//	Hand feed button
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 17,
			kPrintOptionsDlgRsrcID, 18,
			kPrintOptionsDlgRsrcID, 19,
			kPrintOptionsDlgRsrcID, 20
		},
		/* [7] */
		HMSkipItem 						//	Paper Feed: string
		{
		},
		/* [8] */
		HMSkipItem						//	Sheet Feeder Optiosn: string 
		{
		},
		/* [9] */
		HMSkipItem 						//	Bin 1 string
		{
		},
		/* [10] */
		HMSkipItem 						//	Bin 2 string
		{
		},
		/* [11] */
		HMSkipItem 						// Bin 3 string
		{
		},
		/* [12] */
		HMSkipItem 						// First Sheet from: string
		{
		},
		/* [13] */
		HMSkipItem 						//	Remaining Sheets From: string
		{				
		},
		/* [14] */
		HMStringResItem 				//	Bin 1: first sheet from
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 21,
			kPrintOptionsDlgRsrcID, 22,
			kPrintOptionsDlgRsrcID, 23,
			kPrintOptionsDlgRsrcID, 24
		},
		/* [15] */
		HMStringResItem  				//	Bin 2: first sheet from
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 25,
			kPrintOptionsDlgRsrcID, 26,
			kPrintOptionsDlgRsrcID, 27,
			kPrintOptionsDlgRsrcID, 28
		},
		/* [16] */
		HMStringResItem  				//	Bin 3: first sheet from
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 29,
			kPrintOptionsDlgRsrcID, 30,
			kPrintOptionsDlgRsrcID, 31,
			kPrintOptionsDlgRsrcID, 32
		},
		/* [17] */
		HMStringResItem  				//	Bin 1: remaining sheets from
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 33,
			kPrintOptionsDlgRsrcID, 34,
			kPrintOptionsDlgRsrcID, 35,
			kPrintOptionsDlgRsrcID, 36
		},
		/* [18] */
		HMStringResItem   				//	Bin 2: remaining sheets from
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 37,
			kPrintOptionsDlgRsrcID, 38,
			kPrintOptionsDlgRsrcID, 39,
			kPrintOptionsDlgRsrcID, 40
		},
		/* [19] */
		HMStringResItem   				//	Bin 3: remaining sheets from
		{
			{0, 0},
			{0, 0, 0, 0},
			kPrintOptionsDlgRsrcID, 41,
			kPrintOptionsDlgRsrcID, 42,
			kPrintOptionsDlgRsrcID, 43,
			kPrintOptionsDlgRsrcID, 44
		},
		/* [20] */
		HMSkipItem 							//	Please note: string
		{
		},
		/* [21] */
		HMSkipItem 							//	Underline string
		{
		},
		/* [22] */
		HMSkipItem 							//	Underline string
		{
		},
		/* [23] */
		HMSkipItem 							//	Version string
		{
		},
		/* [24] */
		HMSkipItem 							//	Default outline of the OK button
		{
		}
	}
};


// This 'STR#' resource contains the help strings that are referenced in the 'hdlg' resource
// that corresponds to the PrStlDialog.

resource 'STR#' (gxStlDialogResID, sysheap, purgeable) 
{
	{	/* array StringArray: 60 elements */
		/* [1] */	"Missing enabled item in hdlg -8192",
		/* [2] */	"Missing disabled item in hdlg -8192",
		/* [3] */	"Missing checked item in hdlg -8192",
		/* [4] */	"Missing other item in hdlg -8192",
		/* [5] */	"To confirm the settings in this box and close the box, click this button (or press the Return or Enter key).",
		/* [6] */	"",
		/* [7] */	"",
		/* [8] */	"",
		/* [9] */	"To cancel any changes to the settings in this box and close the box, click this button.",
		/* [10] */	"",
		/* [11] */	"",
		/* [12] */	"",
		/* [13] */	"To print your document with standard U.S. letter paper (8 1/2 by 11 inches), click this button.",
		/* [14] */	"",
		/* [15] */	"When this button is selected, your document is set up for standard U.S. letter paper (8 1/2 by 11 inches).",
		/* [16] */	"",
		/* [17] */	"To print your document with A4 paper (a standard European letter size, 8 1/2 by 11 2/3 inches), click this button.",
		/* [18] */	"",
		/* [19] */	"When this button is selected, your document is set up for A4 paper (a standard European letter size, 8 1/2 by 11 2/3 inches).",
		/* [20] */	"",
		/* [21] */	"To print your document with U.S. legal paper (8 1/2 by 14 inches), click this button.",
		/* [22] */	"",
		/* [23] */	"When this button is selected, your document is set up for U.S. legal paper (8 1/2 by 14 inches).",
		/* [24] */	"",
		/* [25] */	"To print your document with international fanfold paper (8 1/4 by 12 inches), click this button.",
		/* [26] */	"",
		/* [27] */	"When this button is selected, your document is set up for international fanfold paper (8 1/4 by 12 inches).",
		/* [28] */	"",
		/* [29] */	"To print your document with standard computer paper (15 by 11 inches), click this button.",
		/* [30] */	"",
		/* [31] */	"When this button is selected, your document is set up for standard computer paper (15 by 11 inches).",
		/* [32] */	"",
		/* [33] */	"To print on number 10 (U.S. business-size) envelopes, click this button.",
		/* [34] */	"",
		/* [35] */	"When this button is selected, your document is set up for number 10 (U.S. business-size) envelopes.",
		/* [36] */	"",
		/* [37] */	"To set up your document for portrait (tall) printing, click this icon.",
		/* [38] */	"",
		/* [39] */	"When this icon is selected, your document is set up for portrait (tall) printing.",
		/* [40] */	"",
		/* [41] */	"To set up your document for landscape (wide) printing, click this icon.",
		/* [42] */	"",
		/* [43] */	"When this icon is selected, your document is set up for landscape (wide) printing.",
		/* [44] */	"",
		/* [45] */	"To print your document at two-third its actual size, click this button.",
		/* [46] */	"To print your document at two-third its actual size, click this button.  Not available because the application program you"
						" are using always prints images at actual size.",
		/* [47] */	"When this button is selected, your document is printed at two-third its actual size.",
		/* [48] */	"",
		/* [49] */	"To print your document at one-third its actual size, click this button.",
		/* [50] */	"To print your document at one-third its actual size, click this button.  Not available because the application program you"
						"are using always prints images at actual size.",
		/* [51] */	"When this button is selected, your document is printed at one-third its actual size.",
		/* [52] */	"",
		/* [53] */	"To print the document at its full size (with no reduction), click this button.",
		/* [54] */	"",
		/* [55] */	"When this button is selected, the document is printed at full size (with no reduction).",
		/* [56] */	"",
		/* [57] */	"To print your document without gaps between pages, click this  box.",
		/* [58] */	"",
		/* [59] */	"When this box is checked, your document is printed without gaps between pages.",
		/* [60] */	""
	}
};


// This 'STR#' resource contains the help strings that are referenced in the 'hdlg' resource
// that corresponds to the PrJobDialog.

resource 'STR#' (gxJobDialogResID, sysheap, purgeable) 
{
	{	/* array StringArray: 56 elements */
		/* [1] */	"Missing enabled item in hdlg -8191",
		/* [2] */	"Missing disabled item in hdlg -8191",
		/* [3] */	"Missing checked item in hdlg -8191",
		/* [4] */	"Missing other item in hdlg -8191",
		/* [5] */	"To start printing using the settings in this dialog box, click this button (or press the Return or Enter key).",
		/* [6] */	"",
		/* [7] */	"",
		/* [8] */	"",
		/* [9] */	"To cancel the Print command without saving any changes to settings in the dialog box, click this button.",
		/* [10] */	"",
		/* [11] */	"",
		/* [12] */	"",
		/* [13] */	"To print your document with the highest resolution (216 dots per inch), click this button.  Best quality takes the longest to print.",
		/* [14] */	"",
		/* [15] */	"When this button is selected, your document is printed with the highest resolution (216 dots per inch).  Best quality takes the longest to print.",
		/* [16] */	"",
		/* [17] */	"To print your document with medium resolution (72 dots per inch), click this button.  Faster quality prints more quickly than Best, but more slowly than Draft.",
		/* [18] */	"To print your document with medium resolution (72 dots per inch), click this button.  Not available because the application program you are using always prints at the highest resolution.",
		/* [19] */	"When this button is selected, your document prints with medium resolution (72 dots per inch).  Faster quality prints more quickly than Best, but more slowly than Draft.",
		/* [20] */	"",
		/* [21] */	"To get a quick printout, click this button.  Text only is printed, using one font and font size, with plain, bold, and underline styles only.",
		/* [22] */	"",
		/* [23] */	"When this button is selected, you get a quick printout containing text only, using one font and font size, with plain, bold, and underline styles only.",
		/* [24] */	"",
		/* [25] */	"To print all the pages in the document, click this button.",
		/* [26] */	"",
		/* [27] */	"When this button is selected, all the pages in the document are printed.",
		/* [28] */	"",
		/* [29] */	"To print a range of pages, click this button and type the first and last page numbers in the boxes to the right.  To print one page, type its number in both boxes.",
		/* [30] */	"",
		/* [31] */	"When this button is selected, only the page range specified in the boxes to the right is printed.",
		/* [32] */	"",
		/* [33] */	"",
		/* [34] */	"To specify a range of pages to be printed, type the number of the first page here.",
		/* [35] */	"",
		/* [36] */	"",
		/* [37] */	"",
		/* [38] */	"To specify a range of pages to be printed, type the number of the last page here.",
		/* [39] */	"",
		/* [40] */	"",
		/* [41] */	"",
		/* [42] */	"Type the number of copies to be printed here.",
		/* [43] */	"",
		/* [44] */	"",
		/* [45] */	"To set the printer to print both left-to-right and right-to-left, click this button.  Printing is about 20% faster, but there may be slight problems with alignment.",
		/* [46] */	"",
		/* [47] */	"When this button is selected, the printer prints both left-to-right and right-to-left.  Printing is about 20% faster, but there may be slight problems with alignment.",
		/* [48] */	"",
		/* [49] */	"To set the printer to print left-to-right only, click this button.  Printing is slower than bidirectional printing, but print quality is higher.",
		/* [50] */	"",
		/* [51] */	"When this button is selected, the printer prints left-to-right only.  Printing is slower than bidirectional printing, but print quality is higher.",
		/* [52] */	"",
		/* [53] */	"To set paper feed options and sheet feeder options, click this button.",
		/* [54] */	"",
		/* [55] */	"",
		/* [56] */	""
	}
};


// This 'STR#' resource contains the help strings that are referenced in the 'hdlg' resource
// that corresponds to the Print Options dialog.

resource 'STR#' (kPrintOptionsDlgRsrcID, sysheap, purgeable) 
{
	{	/* array StringArray: 44 elements */
		/* [1] */ "Missing enabled item in hdlg -8138",
		/* [2] */ "Missing disabled item in hdlg -8138",
		/* [3] */ "Missing checked item in hdlg -8138",
		/* [4] */ "Missing other item in hdlg -8138",
		/* [5] */ "To confirm the settings in this box and close the box, click this button (or press the Return or Enter key).",
		/* [6] */ "",
		/* [7] */ "",
		/* [8] */ "",
		/* [9] */ "To cancel any changes you have made to the settings in this box and close the box, click this button.",
		/* [10] */ "",
		/* [11] */ "",
		/* [12] */ "",
		/* [13] */ "To print on paper automatically fed into the printer, click this button.",
		/* [14] */ "",
		/* [15] */ "When this button is selected, the printer uses paper which is automatically fed into the printer.",
		/* [16] */ "",
		/* [17] */ "To print on paper fed into the printer by hand (one sheet at a time), click this button",
		/* [18] */ "",
		/* [19] */ "When this button is selected, the ImageWriter will pause before printing each page so that you can insert a new sheet of paper.",
		/* [20] */ "",
		/* [21] */ "To print the first page of your document on paper from bin 1, click this button.",
		/* [22] */ "",
		/* [23] */ "When this button is selected, the first page of your document is printed on paper from bin 1.",
		/* [24] */ "",
		/* [25] */ "To print the first page of your document on paper from bin 2, click this button.",
		/* [26] */ "",
		/* [27] */ "When this button is selected, the first page of your document is printed on paper from bin 2.",
		/* [28] */ "",
		/* [29] */ "To print the first page of your document on paper from bin 3, click this button.",
		/* [30] */ "",
		/* [31] */ "When this button is selected, the first page of your document is printed on paper from bin 3.",
		/* [32] */ "",
		/* [33] */ "To print every page except the first on paper from bin 1, click this button.",
		/* [34] */ "",
		/* [35] */ "When this button is selected, every page except the first is printed on paper from bin 1.",
		/* [36] */ "",
		/* [37] */ "To print every page except the first on paper from bin 2, click this button.",
		/* [38] */ "",
		/* [39] */ "When this button is selected, every page except the first is printed on paper from bin 2.",
		/* [40] */ "",
		/* [41] */ "To print every page except the first on paper from bin 3, click this button.",
		/* [42] */ "",
		/* [43] */ "When this button is selected, every page except the first is printed on paper from bin 3.",
		/* [44] */ ""
	}
};
